@@ -35,27 +35,31 @@ Examples of pyodbc connection string URLs:
 
     dsn=mydsn;UID=user;PWD=pass;LANGUAGE=us_english
 
-* ``mssql+pyodbc://user:pass@host/db`` - connects using a connection string
-  dynamically created that would appear like::
+* ``mssql+pyodbc://user:pass@host/db`` - connects using a connection 
+  that would appear like::
 
     DRIVER={SQL Server};Server=host;Database=db;UID=user;PWD=pass
 
 * ``mssql+pyodbc://user:pass@host:123/db`` - connects using a connection
-  string that is dynamically created, which also includes the port
-  information using the comma syntax. If your connection string
-  requires the port information to be passed as a ``port`` keyword
-  see the next example. This will create the following connection
-  string::
+  string which includes the port
+  information using the comma syntax. This will create the following 
+  connection string::
 
     DRIVER={SQL Server};Server=host,123;Database=db;UID=user;PWD=pass
 
 * ``mssql+pyodbc://user:pass@host/db?port=123`` - connects using a connection
-  string that is dynamically created that includes the port
+  string that includes the port
   information as a separate ``port`` keyword. This will create the
   following connection string::
 
     DRIVER={SQL Server};Server=host;Database=db;UID=user;PWD=pass;port=123
 
+* ``mssql+pyodbc://user:pass@host/db?driver=MyDriver`` - connects using a connection
+  string that includes a custom
+  ODBC driver name.  This will create the following connection string::
+
+    DRIVER={MyDriver};Server=host;Database=db;UID=user;PWD=pass
+
 If you require a connection string that is outside the options
 presented above, use the ``odbc_connect`` keyword to pass in a
 urlencoded connection string. What gets passed in will be urldecoded
@@ -94,7 +98,12 @@ class _MSNumeric_pyodbc(sqltypes.Numeric):
     """
 
     def bind_processor(self, dialect):
-        super_process = super(_MSNumeric_pyodbc, self).bind_processor(dialect)
+
+        super_process = super(_MSNumeric_pyodbc, self).\
+                        bind_processor(dialect)
+
+        if not dialect._need_decimal_fix:
+            return super_process
 
         def process(value):
             if self.asdecimal and \
@@ -112,31 +121,35 @@ class _MSNumeric_pyodbc(sqltypes.Numeric):
                 return value
         return process
 
+    # these routines needed for older versions of pyodbc.
+    # as of 2.1.8 this logic is integrated.
+
     def _small_dec_to_string(self, value):
         return "%s0.%s%s" % (
                     (value < 0 and '-' or ''),
                     '0' * (abs(value.adjusted()) - 1),
-                    "".join([str(nint) for nint in value._int]))
+                    "".join([str(nint) for nint in value.as_tuple()[1]]))
 
     def _large_dec_to_string(self, value):
+        _int = value.as_tuple()[1]
         if 'E' in str(value):
             result = "%s%s%s" % (
                     (value < 0 and '-' or ''),
-                    "".join([str(s) for s in value._int]),
-                    "0" * (value.adjusted() - (len(value._int)-1)))
+                    "".join([str(s) for s in _int]),
+                    "0" * (value.adjusted() - (len(_int)-1)))
         else:
-            if (len(value._int) - 1) > value.adjusted():
+            if (len(_int) - 1) > value.adjusted():
                 result = "%s%s.%s" % (
                 (value < 0 and '-' or ''),
                 "".join(
-                    [str(s) for s in value._int][0:value.adjusted() + 1]),
+                    [str(s) for s in _int][0:value.adjusted() + 1]),
                 "".join(
-                    [str(s) for s in value._int][value.adjusted() + 1:]))
+                    [str(s) for s in _int][value.adjusted() + 1:]))
             else:
                 result = "%s%s" % (
                 (value < 0 and '-' or ''),
                 "".join(
-                    [str(s) for s in value._int][0:value.adjusted() + 1]))
+                    [str(s) for s in _int][0:value.adjusted() + 1]))
         return result
 
 
@@ -206,5 +219,7 @@ class MSDialect_pyodbc(PyODBCConnector, MSDialect):
         self.description_encoding = description_encoding
         self.use_scope_identity = self.dbapi and \
                         hasattr(self.dbapi.Cursor, 'nextset')
+        self._need_decimal_fix = self.dbapi and \
+                            self._dbapi_version() < (2, 1, 8)
 
 dialect = MSDialect_pyodbc
