@@ -50,6 +50,8 @@ Unicode
 cx_oracle 5 fully supports Python unicode objects.   SQLAlchemy will pass
 all unicode strings directly to cx_oracle, and additionally uses an output
 handler so that all string based result values are returned as unicode as well.
+Generally, the ``NLS_LANG`` environment variable determines the nature
+of the encoding to be used.
 
 Note that this behavior is disabled when Oracle 8 is detected, as it has been 
 observed that issues remain when passing Python unicodes to cx_oracle with Oracle 8.
@@ -127,7 +129,8 @@ from sqlalchemy.engine import base
 from sqlalchemy import types as sqltypes, util, exc, processors
 from datetime import datetime
 import random
-from decimal import Decimal
+import collections
+from sqlalchemy.util.compat import decimal
 import re
 
 class _OracleNumeric(sqltypes.Numeric):
@@ -154,10 +157,10 @@ class _OracleNumeric(sqltypes.Numeric):
                 def to_decimal(value):
                     if value is None:
                         return None
-                    elif isinstance(value, Decimal):
+                    elif isinstance(value, decimal.Decimal):
                         return value
                     else:
-                        return Decimal(fstring % value)
+                        return decimal.Decimal(fstring % value)
                 return to_decimal
             else:
                 if self.precision is None and self.scale is None:
@@ -338,7 +341,7 @@ class OracleExecutionContext_cx_oracle(OracleExecutionContext):
                                                         self.out_parameters[name]
 
     def create_cursor(self):
-        c = self._connection.connection.cursor()
+        c = self._dbapi_connection.cursor()
         if self.dialect.arraysize:
             c.arraysize = self.dialect.arraysize
 
@@ -428,8 +431,8 @@ class ReturningResultProxy(base.FullyBufferedResultProxy):
         return ret
 
     def _buffer_rows(self):
-        return [tuple(self._returning_params["ret_%d" % i] 
-                    for i, c in enumerate(self._returning_params))]
+        return collections.deque([tuple(self._returning_params["ret_%d" % i] 
+                    for i, c in enumerate(self._returning_params))])
 
 class OracleDialect_cx_oracle(OracleDialect):
     execution_ctx_cls = OracleExecutionContext_cx_oracle
@@ -580,15 +583,15 @@ class OracleDialect_cx_oracle(OracleDialect):
             self._detect_decimal = \
                 lambda value: _detect_decimal(value.replace(char, '.'))
             self._to_decimal = \
-                lambda value: Decimal(value.replace(char, '.'))
+                lambda value: decimal.Decimal(value.replace(char, '.'))
 
     def _detect_decimal(self, value):
         if "." in value:
-            return Decimal(value)
+            return decimal.Decimal(value)
         else:
             return int(value)
 
-    _to_decimal = Decimal
+    _to_decimal = decimal.Decimal
 
     def on_connect(self):
         if self.cx_oracle_ver < (5,):
@@ -684,11 +687,18 @@ class OracleDialect_cx_oracle(OracleDialect):
                         for x in connection.connection.version.split('.')
                     )
 
-    def is_disconnect(self, e):
+    def is_disconnect(self, e, connection, cursor):
+        error, = e.args
         if isinstance(e, self.dbapi.InterfaceError):
             return "not connected" in str(e)
+        elif hasattr(error, 'code'):
+            # ORA-00028: your session has been killed
+            # ORA-03114: not connected to ORACLE
+            # ORA-03113: end-of-file on communication channel
+            # ORA-01033: ORACLE initialization or shutdown in progress
+            return error.code in (28, 3114, 3113, 1033)
         else:
-            return "ORA-03114" in str(e) or "ORA-03113" in str(e)
+            return False
 
     def create_xid(self):
         """create a two-phase transaction ID.
