@@ -77,7 +77,7 @@ def _register_attribute(strategy, mapper, useobject,
                 compare_function=compare_function, 
                 useobject=useobject,
                 extension=attribute_ext, 
-                trackparent=useobject, 
+                trackparent=useobject and (prop.single_parent or prop.direction is interfaces.ONETOMANY), 
                 typecallable=typecallable,
                 callable_=callable_, 
                 active_history=active_history,
@@ -748,6 +748,9 @@ class SubqueryLoader(AbstractRelationshipLoader):
         # with polymorphic loading ?
         q._set_entities(q._adapt_col_list(leftmost_attr))
 
+        if q._order_by is False:
+            q._order_by = leftmost_mapper.order_by
+
         # don't need ORDER BY if no limit/offset
         if q._limit is None and q._offset is None:
             q._order_by = None
@@ -800,6 +803,7 @@ class SubqueryLoader(AbstractRelationshipLoader):
             getattr(parent_alias, self.parent._columntoproperty[c].key)
             for c in local_cols
         ]
+
         q = q.order_by(*local_attr)
         q = q.add_columns(*local_attr)
 
@@ -1304,7 +1308,7 @@ class LoadEagerFromAliasOption(PropertyOption):
 
 def single_parent_validator(desc, prop):
     def _do_check(state, value, oldvalue, initiator):
-        if value is not None:
+        if value is not None and initiator.key == prop.key:
             hasparent = initiator.hasparent(attributes.instance_state(value))
             if hasparent and oldvalue is not value: 
                 raise sa_exc.InvalidRequestError(
