@@ -30,7 +30,7 @@ from sqlalchemy.util import pickle
 from sqlalchemy.util.compat import decimal
 from sqlalchemy.sql.visitors import Visitable
 from sqlalchemy import util
-from sqlalchemy import processors, events
+from sqlalchemy import processors, events, event
 import collections
 default = util.importlater("sqlalchemy.engine", "default")
 
@@ -112,6 +112,25 @@ class TypeEngine(AbstractType):
         """
         return None
 
+    @property
+    def python_type(self):
+        """Return the Python type object expected to be returned
+        by instances of this type, if known.   
+        
+        Basically, for those types which enforce a return type,
+        or are known across the board to do such for all common 
+        DBAPIs (like ``int`` for example), will return that type.
+        
+        If a return type is not defined, raises
+        ``NotImplementedError``.
+        
+        Note that any type also accommodates NULL in SQL which
+        means you can also get back ``None`` from any type
+        in practice.
+
+        """
+        raise NotImplementedError()
+
     def with_variant(self, type_, dialect_name):
         """Produce a new type object that will utilize the given 
         type when applied to the dialect of the given name.
@@ -471,7 +490,7 @@ class TypeDecorator(TypeEngine):
         tt.impl = typedesc
         return tt
 
-    @util.memoized_property
+    @property
     def _type_affinity(self):
         return self.impl._type_affinity
 
@@ -943,53 +962,55 @@ class String(Concatenable, TypeEngine):
         :param length: optional, a length for the column for use in
           DDL statements.  May be safely omitted if no ``CREATE
           TABLE`` will be issued.  Certain databases may require a
-          *length* for use in DDL, and will raise an exception when
-          the ``CREATE TABLE`` DDL is issued.  Whether the value is
+          ``length`` for use in DDL, and will raise an exception when
+          the ``CREATE TABLE`` DDL is issued if a ``VARCHAR``
+          with no length is included.  Whether the value is
           interpreted as bytes or characters is database specific.
 
-        :param convert_unicode: defaults to False.  If True, the 
-          type will do what is necessary in order to accept 
-          Python Unicode objects as bind parameters, and to return
-          Python Unicode objects in result rows.   This may
-          require SQLAlchemy to explicitly coerce incoming Python 
-          unicodes into an encoding, and from an encoding 
-          back to Unicode, or it may not require any interaction
-          from SQLAlchemy at all, depending on the DBAPI in use.
+        :param convert_unicode: When set to ``True``, the 
+          :class:`.String` type will assume that
+          input is to be passed as Python ``unicode`` objects,
+          and results returned as Python ``unicode`` objects.
+          If the DBAPI in use does not support Python unicode
+          (which is fewer and fewer these days), SQLAlchemy
+          will encode/decode the value, using the 
+          value of the ``encoding`` parameter passed to 
+          :func:`.create_engine` as the encoding.
+          
+          When using a DBAPI that natively supports Python
+          unicode objects, this flag generally does not 
+          need to be set.  For columns that are explicitly
+          intended to store non-ASCII data, the :class:`.Unicode`
+          or :class:`UnicodeText` 
+          types should be used regardless, which feature
+          the same behavior of ``convert_unicode`` but 
+          also indicate an underlying column type that
+          directly supports unicode, such as ``NVARCHAR``.
 
-          When SQLAlchemy performs the encoding/decoding, 
-          the encoding used is configured via
-          :attr:`~sqlalchemy.engine.base.Dialect.encoding`, which
-          defaults to `utf-8`.
+          For the extremely rare case that Python ``unicode``
+          is to be encoded/decoded by SQLAlchemy on a backend
+          that does natively support Python ``unicode``,
+          the value ``force`` can be passed here which will
+          cause SQLAlchemy's encode/decode services to be
+          used unconditionally.
 
-          The "convert_unicode" behavior can also be turned on
-          for all String types by setting 
-          :attr:`sqlalchemy.engine.base.Dialect.convert_unicode`
-          on create_engine().
-
-          To instruct SQLAlchemy to perform Unicode encoding/decoding
-          even on a platform that already handles Unicode natively,
-          set convert_unicode='force'.  This will incur significant
-          performance overhead when fetching unicode result columns.
-
-        :param assert_unicode: Deprecated.  A warning is raised in all cases
-          when a non-Unicode object is passed when SQLAlchemy would coerce
-          into an encoding (note: but **not** when the DBAPI handles unicode
-          objects natively). To suppress or raise this warning to an error,
-          use the Python warnings filter documented at:
-          http://docs.python.org/library/warnings.html
+        :param assert_unicode: Deprecated.  A warning is emitted 
+          when a non-``unicode`` object is passed to the 
+          :class:`.Unicode` subtype of :class:`.String`, 
+          or the :class:`.UnicodeText` subtype of :class:`.Text`.   
+          See :class:`.Unicode` for information on how to 
+          control this warning.
 
         :param unicode_error: Optional, a method to use to handle Unicode
-          conversion errors. Behaves like the 'errors' keyword argument to
-          the standard library's string.decode() functions.   This flag
-          requires that `convert_unicode` is set to `"force"` - otherwise,
+          conversion errors. Behaves like the ``errors`` keyword argument to
+          the standard library's ``string.decode()`` functions.   This flag
+          requires that ``convert_unicode`` is set to ``force`` - otherwise,
           SQLAlchemy is not guaranteed to handle the task of unicode
           conversion.   Note that this flag adds significant performance
           overhead to row-fetching operations for backends that already
           return unicode objects natively (which most DBAPIs do).  This
-          flag should only be used as an absolute last resort for reading
-          strings from a column with varied or corrupted encodings,
-          which only applies to databases that accept invalid encodings 
-          in the first place (i.e. MySQL.  *not* PG, Sqlite, etc.)
+          flag should only be used as a last resort for reading
+          strings from a column with varied or corrupted encodings.
 
         """
         if unicode_error is not None and convert_unicode != 'force':
@@ -1069,6 +1090,13 @@ class String(Concatenable, TypeEngine):
         else:
             return None
 
+    @property
+    def python_type(self):
+        if self.convert_unicode:
+            return unicode
+        else:
+            return str
+
     def get_dbapi_type(self, dbapi):
         return dbapi.STRING
 
@@ -1083,37 +1111,57 @@ class Text(String):
     __visit_name__ = 'text'
 
 class Unicode(String):
-    """A variable length Unicode string.
+    """A variable length Unicode string type.
 
-    The ``Unicode`` type is a :class:`.String` which converts Python
-    ``unicode`` objects (i.e., strings that are defined as
-    ``u'somevalue'``) into encoded bytestrings when passing the value
-    to the database driver, and similarly decodes values from the
-    database back into Python ``unicode`` objects.
+    The :class:`.Unicode` type is a :class:`.String` subclass
+    that assumes input and output as Python ``unicode`` data,
+    and in that regard is equivalent to the usage of the
+    ``convert_unicode`` flag with the :class:`.String` type.
+    However, unlike plain :class:`.String`, it also implies an 
+    underlying column type that is explicitly supporting of non-ASCII
+    data, such as ``NVARCHAR`` on Oracle and SQL Server.
+    This can impact the output of ``CREATE TABLE`` statements 
+    and ``CAST`` functions at the dialect level, and can 
+    also affect the handling of bound parameters in some
+    specific DBAPI scenarios.
+    
+    The encoding used by the :class:`.Unicode` type is usually
+    determined by the DBAPI itself; most modern DBAPIs 
+    feature support for Python ``unicode`` objects as bound
+    values and result set values, and the encoding should
+    be configured as detailed in the notes for the target
+    DBAPI in the :ref:`dialect_toplevel` section.
+    
+    For those DBAPIs which do not support, or are not configured
+    to accommodate Python ``unicode`` objects
+    directly, SQLAlchemy does the encoding and decoding
+    outside of the DBAPI.   The encoding in this scenario 
+    is determined by the ``encoding`` flag passed to 
+    :func:`.create_engine`.
 
-    It's roughly equivalent to using a ``String`` object with
-    ``convert_unicode=True``, however
-    the type has other significances in that it implies the usage 
-    of a unicode-capable type being used on the backend, such as NVARCHAR.
-    This may affect what type is emitted when issuing CREATE TABLE
-    and also may effect some DBAPI-specific details, such as type
-    information passed along to ``setinputsizes()``.
-
-    When using the ``Unicode`` type, it is only appropriate to pass
-    Python ``unicode`` objects, and not plain ``str``.  If a
-    bytestring (``str``) is passed, a runtime warning is issued.  If
-    you notice your application raising these warnings but you're not
-    sure where, the Python ``warnings`` filter can be used to turn
-    these warnings into exceptions which will illustrate a stack
-    trace::
+    When using the :class:`.Unicode` type, it is only appropriate 
+    to pass Python ``unicode`` objects, and not plain ``str``.
+    If a plain ``str`` is passed under Python 2, a warning
+    is emitted.  If you notice your application emitting these warnings but 
+    you're not sure of the source of them, the Python 
+    ``warnings`` filter, documented at 
+    http://docs.python.org/library/warnings.html, 
+    can be used to turn these warnings into exceptions 
+    which will illustrate a stack trace::
 
       import warnings
       warnings.simplefilter('error')
 
-    Bytestrings sent to and received from the database are encoded
-    using the dialect's
-    :attr:`~sqlalchemy.engine.base.Dialect.encoding`, which defaults
-    to `utf-8`.
+    For an application that wishes to pass plain bytestrings
+    and Python ``unicode`` objects to the ``Unicode`` type
+    equally, the bytestrings must first be decoded into 
+    unicode.  The recipe at :ref:`coerce_to_unicode` illustrates
+    how this is done.
+
+    See also:
+
+        :class:`.UnicodeText` - unlengthed textual counterpart
+        to :class:`.Unicode`.
 
     """
 
@@ -1121,17 +1169,11 @@ class Unicode(String):
 
     def __init__(self, length=None, **kwargs):
         """
-        Create a Unicode-converting String type.
-
-        :param length: optional, a length for the column for use in
-          DDL statements.  May be safely omitted if no ``CREATE
-          TABLE`` will be issued.  Certain databases may require a
-          *length* for use in DDL, and will raise an exception when
-          the ``CREATE TABLE`` DDL is issued.  Whether the value is
-          interpreted as bytes or characters is database specific.
-
-        :param \**kwargs: passed through to the underlying ``String``
-          type.
+        Create a :class:`.Unicode` object.
+        
+        Parameters are the same as that of :class:`.String`,
+        with the exception that ``convert_unicode``
+        defaults to ``True``.
 
         """
         kwargs.setdefault('convert_unicode', True)
@@ -1139,13 +1181,14 @@ class Unicode(String):
         super(Unicode, self).__init__(length=length, **kwargs)
 
 class UnicodeText(Text):
-    """An unbounded-length Unicode string.
+    """An unbounded-length Unicode string type.
 
     See :class:`.Unicode` for details on the unicode
     behavior of this object.
 
-    Like ``Unicode``, usage the ``UnicodeText`` type implies a 
-    unicode-capable type being used on the backend, such as NCLOB.
+    Like :class:`.Unicode`, usage the :class:`.UnicodeText` type implies a 
+    unicode-capable type being used on the backend, such as 
+    ``NCLOB``, ``NTEXT``.
 
     """
 
@@ -1155,12 +1198,9 @@ class UnicodeText(Text):
         """
         Create a Unicode-converting Text type.
 
-        :param length: optional, a length for the column for use in
-          DDL statements.  May be safely omitted if no ``CREATE
-          TABLE`` will be issued.  Certain databases may require a
-          *length* for use in DDL, and will raise an exception when
-          the ``CREATE TABLE`` DDL is issued.  Whether the value is
-          interpreted as bytes or characters is database specific.
+        Parameters are the same as that of :class:`.Text`,
+        with the exception that ``convert_unicode``
+        defaults to ``True``.
 
         """
         kwargs.setdefault('convert_unicode', True)
@@ -1176,6 +1216,10 @@ class Integer(_DateAffinity, TypeEngine):
     def get_dbapi_type(self, dbapi):
         return dbapi.NUMBER
 
+    @property
+    def python_type(self):
+        return int
+
     @util.memoized_property
     def _expression_adaptations(self):
         # TODO: need a dictionary object that will
@@ -1311,6 +1355,13 @@ class Numeric(_DateAffinity, TypeEngine):
     def get_dbapi_type(self, dbapi):
         return dbapi.NUMBER
 
+    @property
+    def python_type(self):
+        if self.asdecimal:
+            return decimal.Decimal
+        else:
+            return float
+
     def bind_processor(self, dialect):
         if dialect.supports_native_decimal:
             return None
@@ -1395,9 +1446,17 @@ class Float(Numeric):
           defaults to ``False``.   Note that setting this flag to ``True``
           results in floating point conversion.
 
+        :param \**kwargs: deprecated.  Additional arguments here are ignored
+         by the default :class:`.Float` type.  For database specific 
+         floats that support additional arguments, see that dialect's 
+         documentation for details, such as :class:`sqlalchemy.dialects.mysql.FLOAT`.
+         
         """
         self.precision = precision
         self.asdecimal = asdecimal
+        if kwargs:
+            util.warn_deprecated("Additional keyword arguments "
+                                "passed to Float ignored.")
 
     def result_processor(self, dialect, coltype):
         if self.asdecimal:
@@ -1443,11 +1502,23 @@ class DateTime(_DateAffinity, TypeEngine):
     __visit_name__ = 'datetime'
 
     def __init__(self, timezone=False):
+        """Construct a new :class:`.DateTime`.
+        
+        :param timezone: boolean.  If True, and supported by the
+        backend, will produce 'TIMESTAMP WITH TIMEZONE'. For backends
+        that don't support timezone aware timestamps, has no
+        effect.
+        
+        """
         self.timezone = timezone
 
     def get_dbapi_type(self, dbapi):
         return dbapi.DATETIME
 
+    @property
+    def python_type(self):
+        return dt.datetime
+
     @util.memoized_property
     def _expression_adaptations(self):
         return {
@@ -1469,6 +1540,10 @@ class Date(_DateAffinity,TypeEngine):
     def get_dbapi_type(self, dbapi):
         return dbapi.DATETIME
 
+    @property
+    def python_type(self):
+        return dt.date
+
     @util.memoized_property
     def _expression_adaptations(self):
         return {
@@ -1505,6 +1580,10 @@ class Time(_DateAffinity,TypeEngine):
     def get_dbapi_type(self, dbapi):
         return dbapi.DATETIME
 
+    @property
+    def python_type(self):
+        return dt.time
+
     @util.memoized_property
     def _expression_adaptations(self):
         return {
@@ -1525,6 +1604,14 @@ class _Binary(TypeEngine):
     def __init__(self, length=None):
         self.length = length
 
+    @property
+    def python_type(self):
+        # Py3K
+        #return bytes
+        # Py2K
+        return str
+        # end Py2K
+
     # Python 3 - sqlite3 doesn't need the `Binary` conversion
     # here, though pg8000 does to indicate "bytea"
     def bind_processor(self, dialect):
@@ -1621,26 +1708,45 @@ class SchemaType(events.SchemaEventTarget):
         self.schema = kw.pop('schema', None)
         self.metadata = kw.pop('metadata', None)
         if self.metadata:
-            self.metadata.append_ddl_listener('before-create',
-                    util.portable_instancemethod(self._on_metadata_create))
-            self.metadata.append_ddl_listener('after-drop',
-                    util.portable_instancemethod(self._on_metadata_drop))
+            event.listen(
+                self.metadata,
+                "before_create",
+                util.portable_instancemethod(self._on_metadata_create)
+            )
+            event.listen(
+                self.metadata,
+                "after_drop",
+                util.portable_instancemethod(self._on_metadata_drop)
+            )
 
     def _set_parent(self, column):
         column._on_table_attach(util.portable_instancemethod(self._set_table))
 
     def _set_table(self, column, table):
-        table.append_ddl_listener('before-create',
-                                  util.portable_instancemethod(
-                                        self._on_table_create))
-        table.append_ddl_listener('after-drop',
-                                  util.portable_instancemethod(
-                                        self._on_table_drop))
+        event.listen(
+            table,
+            "before_create",
+              util.portable_instancemethod(
+                    self._on_table_create)
+        )
+        event.listen(
+            table,
+            "after_drop",
+            util.portable_instancemethod(self._on_table_drop)
+        )
         if self.metadata is None:
-            table.metadata.append_ddl_listener('before-create',
-                    util.portable_instancemethod(self._on_metadata_create))
-            table.metadata.append_ddl_listener('after-drop',
-                    util.portable_instancemethod(self._on_metadata_drop))
+            # TODO: what's the difference between self.metadata
+            # and table.metadata here ?
+            event.listen(
+                table.metadata,
+                "before_create",
+                util.portable_instancemethod(self._on_metadata_create)
+            )
+            event.listen(
+                table.metadata,
+                "after_drop",
+                util.portable_instancemethod(self._on_metadata_drop)
+            )
 
     @property
     def bind(self):
@@ -1664,25 +1770,25 @@ class SchemaType(events.SchemaEventTarget):
         if t.__class__ is not self.__class__ and isinstance(t, SchemaType):
             t.drop(bind=bind, checkfirst=checkfirst)
 
-    def _on_table_create(self, event, target, bind, **kw):
+    def _on_table_create(self, target, bind, **kw):
         t = self.dialect_impl(bind.dialect)
         if t.__class__ is not self.__class__ and isinstance(t, SchemaType):
-            t._on_table_create(event, target, bind, **kw)
+            t._on_table_create(target, bind, **kw)
 
-    def _on_table_drop(self, event, target, bind, **kw):
+    def _on_table_drop(self, target, bind, **kw):
         t = self.dialect_impl(bind.dialect)
         if t.__class__ is not self.__class__ and isinstance(t, SchemaType):
-            t._on_table_drop(event, target, bind, **kw)
+            t._on_table_drop(target, bind, **kw)
 
-    def _on_metadata_create(self, event, target, bind, **kw):
+    def _on_metadata_create(self, target, bind, **kw):
         t = self.dialect_impl(bind.dialect)
         if t.__class__ is not self.__class__ and isinstance(t, SchemaType):
-            t._on_metadata_create(event, target, bind, **kw)
+            t._on_metadata_create(target, bind, **kw)
 
-    def _on_metadata_drop(self, event, target, bind, **kw):
+    def _on_metadata_drop(self, target, bind, **kw):
         t = self.dialect_impl(bind.dialect)
         if t.__class__ is not self.__class__ and isinstance(t, SchemaType):
-            t._on_metadata_drop(event, target, bind, **kw)
+            t._on_metadata_drop(target, bind, **kw)
 
 class Enum(String, SchemaType):
     """Generic Enum Type.
@@ -1692,6 +1798,12 @@ class Enum(String, SchemaType):
 
     By default, uses the backend's native ENUM type if available, 
     else uses VARCHAR + a CHECK constraint.
+    
+    See also:
+    
+        :class:`~.postgresql.ENUM` - PostgreSQL-specific type,
+        which has additional functionality.
+        
     """
 
     __visit_name__ = 'enum'
@@ -1940,6 +2052,10 @@ class Boolean(TypeEngine, SchemaType):
                     )
         table.append_constraint(e)
 
+    @property
+    def python_type(self):
+        return bool
+
     def bind_processor(self, dialect):
         if dialect.supports_native_boolean:
             return None
@@ -2006,6 +2122,10 @@ class Interval(_DateAffinity, TypeDecorator):
                         day_precision=self.day_precision,
                         **kw)
 
+    @property
+    def python_type(self):
+        return dt.timedelta
+
     def bind_processor(self, dialect):
         impl_processor = self.impl.bind_processor(dialect)
         epoch = self.epoch
