@@ -1,13 +1,12 @@
-from sqlalchemy.test.testing import assert_raises, assert_raises_message
+from test.lib.testing import assert_raises, assert_raises_message
 import copy, threading
 from sqlalchemy import util, sql, exc
-from sqlalchemy.test import TestBase
-from sqlalchemy.test.testing import eq_, is_, ne_
-from sqlalchemy.test.util import gc_collect, picklers
+from test.lib.testing import eq_, is_, ne_, fails_if
+from test.lib.util import gc_collect, picklers
 from sqlalchemy.util import classproperty
+from test.lib import fixtures
 
-
-class OrderedDictTest(TestBase):
+class OrderedDictTest(fixtures.TestBase):
     def test_odict(self):
         o = util.OrderedDict()
         o['a'] = 1
@@ -71,7 +70,7 @@ class OrderedDictTest(TestBase):
         o3 = copy.copy(o)
         eq_(o3.keys(), o.keys())
 
-class OrderedSetTest(TestBase):
+class OrderedSetTest(fixtures.TestBase):
     def test_mutators_against_iter(self):
         # testing a set modified against an iterator
         o = util.OrderedSet([3,2, 4, 5])
@@ -80,14 +79,14 @@ class OrderedSetTest(TestBase):
         eq_(o.intersection(iter([3,4, 6])), util.OrderedSet([3, 4]))
         eq_(o.union(iter([3,4, 6])), util.OrderedSet([2, 3, 4, 5, 6]))
 
-class FrozenDictTest(TestBase):
+class FrozenDictTest(fixtures.TestBase):
     def test_serialize(self):
-        d = util.frozendict({1:2, 3:4})
+        d = util.immutabledict({1:2, 3:4})
         for loads, dumps in picklers():
             print loads(dumps(d))
-        
 
-class MemoizedAttrTest(TestBase):
+
+class MemoizedAttrTest(fixtures.TestBase):
     def test_memoized_property(self):
         val = [20]
         class Foo(object):
@@ -96,7 +95,7 @@ class MemoizedAttrTest(TestBase):
                 v = val[0]
                 val[0] += 1
                 return v
-                
+
         ne_(Foo.bar, None)
         f1 = Foo()
         assert 'bar' not in f1.__dict__
@@ -120,8 +119,8 @@ class MemoizedAttrTest(TestBase):
         eq_(f1.bar(), 20)
         eq_(f1.bar(), 20)
         eq_(val[0], 21)
-        
-class ColumnCollectionTest(TestBase):
+
+class ColumnCollectionTest(fixtures.TestBase):
     def test_in(self):
         cc = sql.ColumnCollection()
         cc.add(sql.column('col1'))
@@ -149,11 +148,9 @@ class ColumnCollectionTest(TestBase):
         assert (cc1==cc2).compare(c1 == c2)
         assert not (cc1==cc3).compare(c2 == c3)
 
+class LRUTest(fixtures.TestBase):
 
-
-class LRUTest(TestBase):
-
-    def test_lru(self):                
+    def test_lru(self):
         class item(object):
             def __init__(self, id):
                 self.id = id
@@ -193,12 +190,12 @@ class LRUTest(TestBase):
         l[25] = i2
         assert 25 in l
         assert l[25] is i2
-        
+
 
 class ImmutableSubclass(str):
     pass
 
-class FlattenIteratorTest(TestBase):
+class FlattenIteratorTest(fixtures.TestBase):
 
     def test_flatten(self):
         assert list(util.flatten_iterator([[1, 2, 3], [4, 5, 6], 7,
@@ -207,8 +204,9 @@ class FlattenIteratorTest(TestBase):
     def test_str_with_iter(self):
         """ensure that a str object with an __iter__ method (like in
         PyPy) is not interpreted as an iterable.
-        
+
         """
+
         class IterString(str):
             def __iter__(self):
                 return iter(self + '')
@@ -216,7 +214,7 @@ class FlattenIteratorTest(TestBase):
         assert list(util.flatten_iterator([IterString('asdf'),
                     [IterString('x'), IterString('y')]])) == ['asdf',
                 'x', 'y']
-                
+
 class HashOverride(object):
     def __init__(self, value=None):
         self.value = value
@@ -237,7 +235,7 @@ class EqOverride(object):
             return self.value != other.value
         else:
             return True
-            
+
 class HashEqOverride(object):
     def __init__(self, value=None):
         self.value = value
@@ -255,7 +253,7 @@ class HashEqOverride(object):
             return True
 
 
-class IdentitySetTest(TestBase):
+class IdentitySetTest(fixtures.TestBase):
     def assert_eq(self, identityset, expected_iterable):
         expected = sorted([id(o) for o in expected_iterable])
         found = sorted([id(o) for o in identityset])
@@ -292,6 +290,19 @@ class IdentitySetTest(TestBase):
                 ids.add(data[i])
             self.assert_eq(ids, data)
 
+    def test_dunder_sub(self):
+        IdentitySet = util.IdentitySet
+        o1, o2, o3 = object(), object(), object()
+        ids1 = IdentitySet([o1])
+        ids2 = IdentitySet([o1, o2, o3])
+        eq_(
+            ids2 - ids1,
+            IdentitySet([o2, o3])
+        )
+        
+        ids2 -= ids1
+        eq_(ids2, IdentitySet([o2, o3]))
+
     def test_basic_sanity(self):
         IdentitySet = util.IdentitySet
 
@@ -372,8 +383,8 @@ class IdentitySetTest(TestBase):
         assert_raises(TypeError, lambda: s1 - os2)
         assert_raises(TypeError, lambda: s1 - [3, 4, 5])
 
-class OrderedIdentitySetTest(TestBase):
-    
+class OrderedIdentitySetTest(fixtures.TestBase):
+
     def assert_eq(self, identityset, expected_iterable):
         expected = [id(o) for o in expected_iterable]
         found = [id(o) for o in identityset]
@@ -388,10 +399,10 @@ class OrderedIdentitySetTest(TestBase):
     def test_intersection(self):
         elem = object
         eq_ = self.assert_eq
-        
+
         a, b, c, d, e, f, g = \
                 elem(), elem(), elem(), elem(), elem(), elem(), elem()
-        
+
         s1 = util.OrderedIdentitySet([a, b, c])
         s2 = util.OrderedIdentitySet([d, e, f])
         s3 = util.OrderedIdentitySet([a, d, f, g])
@@ -400,7 +411,7 @@ class OrderedIdentitySetTest(TestBase):
         eq_(s1.union(s2).intersection(s3), [a, d, f])
 
 
-class DictlikeIteritemsTest(TestBase):
+class DictlikeIteritemsTest(fixtures.TestBase):
     baseline = set([('a', 1), ('b', 2), ('c', 3)])
 
     def _ok(self, instance):
@@ -428,7 +439,7 @@ class DictlikeIteritemsTest(TestBase):
         d = UserDict.UserDict(a=1,b=2,c=3)
         self._ok(d)
     # end Py2K
-    
+
     def test_object(self):
         self._notok(object())
 
@@ -477,7 +488,7 @@ class DictlikeIteritemsTest(TestBase):
         self._notok(duck6())
 
 
-class DuckTypeCollectionTest(TestBase):
+class DuckTypeCollectionTest(fixtures.TestBase):
     def test_sets(self):
         # Py2K
         import sets
@@ -488,7 +499,7 @@ class DuckTypeCollectionTest(TestBase):
 
         class ForcedSet(list):
             __emulates__ = set
-        
+
         for type_ in (set,
                       # Py2K
                       sets.Set,
@@ -508,7 +519,7 @@ class DuckTypeCollectionTest(TestBase):
             instance = type_()
             is_(util.duck_type_collection(instance), None)
 
-class ArgInspectionTest(TestBase):
+class ArgInspectionTest(fixtures.TestBase):
     def test_get_cls_kwargs(self):
         class A(object):
             def __init__(self, a):
@@ -575,7 +586,7 @@ class ArgInspectionTest(TestBase):
         test(f3)
         test(f4)
 
-class SymbolTest(TestBase):
+class SymbolTest(fixtures.TestBase):
     def test_basic(self):
         sym1 = util.symbol('foo')
         assert sym1.name == 'foo'
@@ -605,7 +616,7 @@ class SymbolTest(TestBase):
             assert rt is sym1
             assert rt is sym2
 
-class WeakIdentityMappingTest(TestBase):
+class WeakIdentityMappingTest(fixtures.TestBase):
     class Data(object):
         pass
 
@@ -798,7 +809,7 @@ class WeakIdentityMappingTest(TestBase):
         eq_(wim._weakrefs, {})
 
 
-class TestFormatArgspec(TestBase):
+class TestFormatArgspec(fixtures.TestBase):
     def test_specs(self):
         def test(fn, wanted, grouped=None):
             if grouped is None:
@@ -862,6 +873,7 @@ class TestFormatArgspec(TestBase):
             'apply_kw': 'a=a, b=b', 'apply_pos': 'a, b' },
            grouped=False)
 
+    @fails_if(lambda: util.pypy, "object.__init__ is introspectable")
     def test_init_grouped(self):
         object_spec = {
             'args': '(self)', 'self_arg': 'self',
@@ -877,6 +889,7 @@ class TestFormatArgspec(TestBase):
         self._test_init(None, object_spec, wrapper_spec, custom_spec)
         self._test_init(True, object_spec, wrapper_spec, custom_spec)
 
+    @fails_if(lambda: util.pypy,  "object.__init__ can be introspected")
     def test_init_bare(self):
         object_spec = {
             'args': 'self', 'self_arg': 'self',
@@ -937,7 +950,94 @@ class TestFormatArgspec(TestBase):
 
         test(O.__init__, custom_spec)
 
-class AsInterfaceTest(TestBase):
+
+class GenericReprTest(fixtures.TestBase):
+    def test_all_positional(self):
+        class Foo(object):
+            def __init__(self, a, b, c):
+                self.a = a
+                self.b = b
+                self.c = c
+        eq_(
+            util.generic_repr(Foo(1, 2, 3)),
+            "Foo(1, 2, 3)"
+        )
+
+    def test_positional_plus_kw(self):
+        class Foo(object):
+            def __init__(self, a, b, c=5, d=4):
+                self.a = a
+                self.b = b
+                self.c = c
+                self.d = d
+        eq_(
+            util.generic_repr(Foo(1, 2, 3, 6)),
+            "Foo(1, 2, c=3, d=6)"
+        )
+
+    def test_kw_defaults(self):
+        class Foo(object):
+            def __init__(self, a=1, b=2, c=3, d=4):
+                self.a = a
+                self.b = b
+                self.c = c
+                self.d = d
+        eq_(
+            util.generic_repr(Foo(1, 5, 3, 7)),
+            "Foo(b=5, d=7)"
+        )
+
+    def test_discard_vargs(self):
+        class Foo(object):
+            def __init__(self, a, b, *args):
+                self.a = a
+                self.b = b
+                self.c, self.d = args[0:2]
+        eq_(
+            util.generic_repr(Foo(1, 2, 3, 4)),
+            "Foo(1, 2)"
+        )
+
+    def test_discard_vargs_kwargs(self):
+        class Foo(object):
+            def __init__(self, a, b, *args, **kw):
+                self.a = a
+                self.b = b
+                self.c, self.d = args[0:2]
+        eq_(
+            util.generic_repr(Foo(1, 2, 3, 4, x=7, y=4)),
+            "Foo(1, 2)"
+        )
+
+    def test_significant_vargs(self):
+        class Foo(object):
+            def __init__(self, a, b, *args):
+                self.a = a
+                self.b = b
+                self.args = args
+        eq_(
+            util.generic_repr(Foo(1, 2, 3, 4)),
+            "Foo(1, 2, 3, 4)"
+        )
+
+    def test_no_args(self):
+        class Foo(object):
+            def __init__(self):
+                pass
+        eq_(
+            util.generic_repr(Foo()),
+            "Foo()"
+        )
+
+    def test_no_init(self):
+        class Foo(object):
+            pass
+        eq_(
+            util.generic_repr(Foo()),
+            "Foo()"
+        )
+
+class AsInterfaceTest(fixtures.TestBase):
 
     class Something(object):
         def _ignoreme(self): pass
@@ -1026,7 +1126,7 @@ class AsInterfaceTest(TestBase):
                       cls=self.Something)
 
 
-class TestClassHierarchy(TestBase):
+class TestClassHierarchy(fixtures.TestBase):
     def test_object(self):
         eq_(set(util.class_hierarchy(object)), set((object,)))
 
@@ -1045,7 +1145,7 @@ class TestClassHierarchy(TestBase):
 
         eq_(set(util.class_hierarchy(A)), set((A, B, C, object)))
         eq_(set(util.class_hierarchy(B)), set((A, B, C, object)))
-    
+
     # Py2K
     def test_oldstyle_mixin(self):
         class A(object):
@@ -1061,9 +1161,9 @@ class TestClassHierarchy(TestBase):
         eq_(set(util.class_hierarchy(Mixin)), set())
         eq_(set(util.class_hierarchy(A)), set((A, B, object)))
     # end Py2K
-        
 
-class TestClassProperty(TestBase):
+
+class TestClassProperty(fixtures.TestBase):
 
     def test_simple(self):
         class A(object):
