@@ -1,77 +1,86 @@
-from sqlalchemy.test.testing import assert_raises, assert_raises_message
+from test.lib.testing import assert_raises, assert_raises_message
 import sqlalchemy as sa
-from sqlalchemy import Integer, PickleType, String
+from sqlalchemy import Integer, PickleType, String, ForeignKey
 import operator
-from sqlalchemy.test import testing
+from test.lib import testing
 from sqlalchemy.util import OrderedSet
 from sqlalchemy.orm import mapper, relationship, create_session, PropComparator, \
-                            synonym, comparable_property, sessionmaker, attributes
+                            synonym, comparable_property, sessionmaker, attributes,\
+                            Session, backref, configure_mappers
 from sqlalchemy.orm.collections import attribute_mapped_collection
 from sqlalchemy.orm.interfaces import MapperOption
-from sqlalchemy.test.testing import eq_, ne_
-from test.orm import _base, _fixtures
-from sqlalchemy.test.schema import Table, Column
+from test.lib.testing import eq_, ne_
+from test.lib import fixtures
+from test.orm import _fixtures
+from sqlalchemy import event, and_
+from test.lib.schema import Table, Column
 
 class MergeTest(_fixtures.FixtureTest):
     """Session.merge() functionality"""
 
     run_inserts = None
 
-    def on_load_tracker(self, cls, canary=None):
+    def load_tracker(self, cls, canary=None):
         if canary is None:
-            def canary(instance):
+            def canary(instance, *args):
                 canary.called += 1
             canary.called = 0
 
-        manager = sa.orm.attributes.manager_of_class(cls)
-        manager.events.add_listener('on_load', canary)
+        event.listen(cls, 'load', canary)
 
         return canary
 
-    @testing.resolve_artifact_names
     def test_transient_to_pending(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
         sess = create_session()
-        on_load = self.on_load_tracker(User)
+        load = self.load_tracker(User)
 
         u = User(id=7, name='fred')
-        eq_(on_load.called, 0)
+        eq_(load.called, 0)
         u2 = sess.merge(u)
-        eq_(on_load.called, 1)
+        eq_(load.called, 1)
         assert u2 in sess
         eq_(u2, User(id=7, name='fred'))
         sess.flush()
         sess.expunge_all()
         eq_(sess.query(User).first(), User(id=7, name='fred'))
 
-    @testing.resolve_artifact_names
     def test_transient_to_pending_no_pk(self):
         """test that a transient object with no PK attribute doesn't trigger a needless load."""
+
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
         sess = create_session()
         u = User(name='fred')
         def go():
             sess.merge(u)
         self.assert_sql_count(testing.db, go, 0)
-        
-    @testing.resolve_artifact_names
+
     def test_transient_to_pending_collection(self):
+        User, Address, addresses, users = (self.classes.User,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users)
+
         mapper(User, users, properties={
             'addresses': relationship(Address, backref='user',
                                   collection_class=OrderedSet)})
         mapper(Address, addresses)
-        on_load = self.on_load_tracker(User)
-        self.on_load_tracker(Address, on_load)
+        load = self.load_tracker(User)
+        self.load_tracker(Address, load)
 
         u = User(id=7, name='fred', addresses=OrderedSet([
             Address(id=1, email_address='fred1'),
             Address(id=2, email_address='fred2'),
             ]))
-        eq_(on_load.called, 0)
+        eq_(load.called, 0)
 
         sess = create_session()
         sess.merge(u)
-        eq_(on_load.called, 3)
+        eq_(load.called, 3)
 
         merged_users = [e for e in sess if isinstance(e, User)]
         eq_(len(merged_users), 1)
@@ -87,10 +96,11 @@ class MergeTest(_fixtures.FixtureTest):
             ]))
         )
 
-    @testing.resolve_artifact_names
     def test_transient_to_persistent(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
-        on_load = self.on_load_tracker(User)
+        load = self.load_tracker(User)
 
         sess = create_session()
         u = User(id=7, name='fred')
@@ -98,20 +108,24 @@ class MergeTest(_fixtures.FixtureTest):
         sess.flush()
         sess.expunge_all()
 
-        eq_(on_load.called, 0)
+        eq_(load.called, 0)
 
         _u2 = u2 = User(id=7, name='fred jones')
-        eq_(on_load.called, 0)
+        eq_(load.called, 0)
         u2 = sess.merge(u2)
         assert u2 is not _u2
-        eq_(on_load.called, 1)
+        eq_(load.called, 1)
         sess.flush()
         sess.expunge_all()
         eq_(sess.query(User).first(), User(id=7, name='fred jones'))
-        eq_(on_load.called, 2)
+        eq_(load.called, 2)
 
-    @testing.resolve_artifact_names
     def test_transient_to_persistent_collection(self):
+        User, Address, addresses, users = (self.classes.User,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users)
+
         mapper(User, users, properties={
             'addresses':relationship(Address,
                         backref='user',
@@ -121,8 +135,8 @@ class MergeTest(_fixtures.FixtureTest):
         })
         mapper(Address, addresses)
 
-        on_load = self.on_load_tracker(User)
-        self.on_load_tracker(Address, on_load)
+        load = self.load_tracker(User)
+        self.load_tracker(Address, load)
 
         u = User(id=7, name='fred', addresses=OrderedSet([
             Address(id=1, email_address='fred1'),
@@ -133,7 +147,7 @@ class MergeTest(_fixtures.FixtureTest):
         sess.flush()
         sess.expunge_all()
 
-        eq_(on_load.called, 0)
+        eq_(load.called, 0)
 
         u = User(id=7, name='fred', addresses=OrderedSet([
             Address(id=3, email_address='fred3'),
@@ -146,7 +160,7 @@ class MergeTest(_fixtures.FixtureTest):
         # 2.,3. merges Address ids 3 & 4, saves into session.
         # 4.,5. loads pre-existing elements in "addresses" collection,
         # marks as deleted, Address ids 1 and 2.
-        eq_(on_load.called, 5)
+        eq_(load.called, 5)
 
         eq_(u,
             User(id=7, name='fred', addresses=OrderedSet([
@@ -163,16 +177,20 @@ class MergeTest(_fixtures.FixtureTest):
             ]))
         )
 
-    @testing.resolve_artifact_names
     def test_detached_to_persistent_collection(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address,
                                  backref='user',
                                  order_by=addresses.c.id,
                                  collection_class=OrderedSet)})
         mapper(Address, addresses)
-        on_load = self.on_load_tracker(User)
-        self.on_load_tracker(Address, on_load)
+        load = self.load_tracker(User)
+        self.load_tracker(Address, load)
 
         a = Address(id=1, email_address='fred1')
         u = User(id=7, name='fred', addresses=OrderedSet([
@@ -188,9 +206,9 @@ class MergeTest(_fixtures.FixtureTest):
         u.addresses.add(Address(id=3, email_address='fred3'))
         u.addresses.remove(a)
 
-        eq_(on_load.called, 0)
+        eq_(load.called, 0)
         u = sess.merge(u)
-        eq_(on_load.called, 4)
+        eq_(load.called, 4)
         sess.flush()
         sess.expunge_all()
 
@@ -199,16 +217,21 @@ class MergeTest(_fixtures.FixtureTest):
                 Address(id=2, email_address='fred2'),
                 Address(id=3, email_address='fred3')])))
 
-    @testing.resolve_artifact_names
     def test_unsaved_cascade(self):
         """Merge of a transient entity with two child transient entities, with a bidirectional relationship."""
 
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+
         mapper(User, users, properties={
             'addresses':relationship(mapper(Address, addresses),
                                  cascade="all", backref="user")
         })
-        on_load = self.on_load_tracker(User)
-        self.on_load_tracker(Address, on_load)
+        load = self.load_tracker(User)
+        self.load_tracker(Address, load)
         sess = create_session()
 
         u = User(id=7, name='fred')
@@ -218,7 +241,7 @@ class MergeTest(_fixtures.FixtureTest):
         u.addresses.append(a2)
 
         u2 = sess.merge(u)
-        eq_(on_load.called, 3)
+        eq_(load.called, 3)
 
         eq_(u,
             User(id=7, name='fred', addresses=[
@@ -236,14 +259,15 @@ class MergeTest(_fixtures.FixtureTest):
         eq_(u2, User(id=7, name='fred', addresses=[
             Address(email_address='foo@bar.com'),
             Address(email_address='hoho@bar.com')]))
-        eq_(on_load.called, 6)
+        eq_(load.called, 6)
 
-    @testing.resolve_artifact_names
     def test_merge_empty_attributes(self):
+        User, dingalings = self.classes.User, self.tables.dingalings
+
         mapper(User, dingalings)
-        
+
         sess = create_session()
-        
+
         # merge empty stuff.  goes in as NULL.
         # not sure what this was originally trying to 
         # test.
@@ -255,17 +279,17 @@ class MergeTest(_fixtures.FixtureTest):
         u2 = User(id=2, data="foo")
         sess.add(u2)
         sess.flush()
-        
+
         # merge User on u2's pk with
         # no "data".
         # value isn't whacked from the destination
         # dict.
         u3 = sess.merge(User(id=2))
         eq_(u3.__dict__['data'], "foo")
-        
+
         # make a change.
         u3.data = 'bar'
-        
+
         # merge another no-"data" user.
         # attribute maintains modified state.
         # (usually autoflush would have happened
@@ -281,12 +305,12 @@ class MergeTest(_fixtures.FixtureTest):
         u5 = User(id=3, data="foo")
         sess.add(u5)
         sess.flush()
-        
+
         # blow it away from u5, but don't
         # mark as expired.  so it would just 
         # be blank.
         del u5.data
-        
+
         # the merge adds expiry to the
         # attribute so that it loads.
         # not sure if I like this - it currently is needed
@@ -300,10 +324,14 @@ class MergeTest(_fixtures.FixtureTest):
         u6.data = None
         u7 = sess.merge(User(id=3))
         assert u6.__dict__['data'] is None
-        
-        
-    @testing.resolve_artifact_names
+
+
     def test_merge_irregular_collection(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses': relationship(
                 mapper(Address, addresses),
@@ -317,15 +345,20 @@ class MergeTest(_fixtures.FixtureTest):
         sess.flush()
         assert u1.addresses.keys() == ['foo@bar.com']
 
-    @testing.resolve_artifact_names
     def test_attribute_cascade(self):
         """Merge of a persistent entity with two child persistent entities."""
 
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+
         mapper(User, users, properties={
             'addresses':relationship(mapper(Address, addresses), backref='user')
         })
-        on_load = self.on_load_tracker(User)
-        self.on_load_tracker(Address, on_load)
+        load = self.load_tracker(User)
+        self.load_tracker(Address, load)
 
         sess = create_session()
 
@@ -348,12 +381,12 @@ class MergeTest(_fixtures.FixtureTest):
         u.name = 'fred2'
         u.addresses[1].email_address = 'hoho@lalala.com'
 
-        eq_(on_load.called, 3)
+        eq_(load.called, 3)
 
         # new session, merge modified data into session
         sess3 = create_session()
         u3 = sess3.merge(u)
-        eq_(on_load.called, 6)
+        eq_(load.called, 6)
 
         # ensure local changes are pending
         eq_(u3, User(id=7, name='fred2', addresses=[
@@ -369,7 +402,7 @@ class MergeTest(_fixtures.FixtureTest):
         eq_(u, User(id=7, name='fred2', addresses=[
             Address(email_address='foo@bar.com'),
             Address(email_address='hoho@lalala.com')]))
-        eq_(on_load.called, 9)
+        eq_(load.called, 9)
 
         # merge persistent object into another session
         sess4 = create_session()
@@ -381,7 +414,7 @@ class MergeTest(_fixtures.FixtureTest):
             sess4.flush()
         # no changes; therefore flush should do nothing
         self.assert_sql_count(testing.db, go, 0)
-        eq_(on_load.called, 12)
+        eq_(load.called, 12)
 
         # test with "dontload" merge
         sess5 = create_session()
@@ -395,7 +428,7 @@ class MergeTest(_fixtures.FixtureTest):
         # but also, load=False wipes out any difference in committed state,
         # so no flush at all
         self.assert_sql_count(testing.db, go, 0)
-        eq_(on_load.called, 15)
+        eq_(load.called, 15)
 
         sess4 = create_session()
         u = sess4.merge(u, load=False)
@@ -405,19 +438,24 @@ class MergeTest(_fixtures.FixtureTest):
             sess4.flush()
         # afafds change flushes
         self.assert_sql_count(testing.db, go, 1)
-        eq_(on_load.called, 18)
+        eq_(load.called, 18)
 
         sess5 = create_session()
         u2 = sess5.query(User).get(u.id)
         eq_(u2.name, 'fred2')
         eq_(u2.addresses[1].email_address, 'afafds')
-        eq_(on_load.called, 21)
+        eq_(load.called, 21)
 
-    @testing.resolve_artifact_names
     def test_no_relationship_cascade(self):
         """test that merge doesn't interfere with a relationship()
            target that specifically doesn't include 'merge' cascade.
         """
+
+        Address, addresses, users, User = (self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users,
+                                self.classes.User)
+
         mapper(Address, addresses, properties={
             'user':relationship(User, cascade="save-update")
         })
@@ -427,35 +465,39 @@ class MergeTest(_fixtures.FixtureTest):
         a1 = Address(email_address="asdf", user=u1)
         sess.add(a1)
         sess.flush()
-        
+
         a2 = Address(id=a1.id, email_address="bar", user=User(name="hoho"))
         a2 = sess.merge(a2)
         sess.flush()
-        
+
         # no expire of the attribute
-        
+
         assert a2.__dict__['user'] is u1
-        
+
         # merge succeeded
         eq_(
             sess.query(Address).all(),
             [Address(id=a1.id, email_address="bar")]
         )
-        
+
         # didn't touch user
         eq_(
             sess.query(User).all(),
             [User(name="fred")]
         )
-        
-    @testing.resolve_artifact_names
+
     def test_one_to_many_cascade(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
 
         mapper(User, users, properties={
             'addresses':relationship(mapper(Address, addresses))})
 
-        on_load = self.on_load_tracker(User)
-        self.on_load_tracker(Address, on_load)
+        load = self.load_tracker(User)
+        self.load_tracker(Address, load)
 
         sess = create_session()
         u = User(name='fred')
@@ -466,53 +508,57 @@ class MergeTest(_fixtures.FixtureTest):
         sess.add(u)
         sess.flush()
 
-        eq_(on_load.called, 0)
+        eq_(load.called, 0)
 
         sess2 = create_session()
         u2 = sess2.query(User).get(u.id)
-        eq_(on_load.called, 1)
+        eq_(load.called, 1)
 
         u.addresses[1].email_address = 'addr 2 modified'
         sess2.merge(u)
         eq_(u2.addresses[1].email_address, 'addr 2 modified')
-        eq_(on_load.called, 3)
+        eq_(load.called, 3)
 
         sess3 = create_session()
         u3 = sess3.query(User).get(u.id)
-        eq_(on_load.called, 4)
+        eq_(load.called, 4)
 
         u.name = 'also fred'
         sess3.merge(u)
-        eq_(on_load.called, 6)
+        eq_(load.called, 6)
         eq_(u3.name, 'also fred')
 
-    @testing.resolve_artifact_names
     def test_many_to_one_cascade(self):
+        Address, addresses, users, User = (self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users,
+                                self.classes.User)
+
         mapper(Address, addresses, properties={
             'user':relationship(User)
         })
         mapper(User, users)
-        
+
         u1 = User(id=1, name="u1")
         a1 =Address(id=1, email_address="a1", user=u1)
         u2 = User(id=2, name="u2")
-        
+
         sess = create_session()
         sess.add_all([a1, u2])
         sess.flush()
-        
+
         a1.user = u2
-        
+
         sess2 = create_session()
         a2 = sess2.merge(a1)
         eq_(
             attributes.get_history(a2, 'user'), 
-            ([u2], (), [attributes.PASSIVE_NO_RESULT])
+            ([u2], (), ())
         )
         assert a2 in sess2.dirty
-        
+
         sess.refresh(a1)
-        
+
         sess2 = create_session()
         a2 = sess2.merge(a1, load=False)
         eq_(
@@ -520,15 +566,20 @@ class MergeTest(_fixtures.FixtureTest):
             ((), [u1], ())
         )
         assert a2 not in sess2.dirty
-        
-    @testing.resolve_artifact_names
+
     def test_many_to_many_cascade(self):
+        items, Order, orders, order_items, Item = (self.tables.items,
+                                self.classes.Order,
+                                self.tables.orders,
+                                self.tables.order_items,
+                                self.classes.Item)
+
 
         mapper(Order, orders, properties={
             'items':relationship(mapper(Item, items), secondary=order_items)})
 
-        on_load = self.on_load_tracker(Order)
-        self.on_load_tracker(Item, on_load)
+        load = self.load_tracker(Order)
+        self.load_tracker(Item, load)
 
         sess = create_session()
 
@@ -546,34 +597,38 @@ class MergeTest(_fixtures.FixtureTest):
         sess.add(o)
         sess.flush()
 
-        eq_(on_load.called, 0)
+        eq_(load.called, 0)
 
         sess2 = create_session()
         o2 = sess2.query(Order).get(o.id)
-        eq_(on_load.called, 1)
+        eq_(load.called, 1)
 
         o.items[1].description = 'item 2 modified'
         sess2.merge(o)
         eq_(o2.items[1].description, 'item 2 modified')
-        eq_(on_load.called,  3)
+        eq_(load.called,  3)
 
         sess3 = create_session()
         o3 = sess3.query(Order).get(o.id)
-        eq_( on_load.called, 4)
+        eq_( load.called, 4)
 
         o.description = 'desc modified'
         sess3.merge(o)
-        eq_(on_load.called, 6)
+        eq_(load.called, 6)
         eq_(o3.description, 'desc modified')
 
-    @testing.resolve_artifact_names
     def test_one_to_one_cascade(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
 
         mapper(User, users, properties={
             'address':relationship(mapper(Address, addresses),uselist = False)
         })
-        on_load = self.on_load_tracker(User)
-        self.on_load_tracker(Address, on_load)
+        load = self.load_tracker(User)
+        self.load_tracker(Address, load)
         sess = create_session()
 
         u = User()
@@ -586,21 +641,25 @@ class MergeTest(_fixtures.FixtureTest):
         sess.add(u)
         sess.flush()
 
-        eq_(on_load.called, 0)
+        eq_(load.called, 0)
 
         sess2 = create_session()
         u2 = sess2.query(User).get(7)
-        eq_(on_load.called, 1)
+        eq_(load.called, 1)
         u2.name = 'fred2'
         u2.address.email_address = 'hoho@lalala.com'
-        eq_(on_load.called, 2)
+        eq_(load.called, 2)
 
         u3 = sess.merge(u2)
-        eq_(on_load.called, 2)
+        eq_(load.called, 2)
         assert u3 is u
 
-    @testing.resolve_artifact_names
     def test_value_to_none(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'address':relationship(mapper(Address, addresses),uselist = False, backref='user')
         })
@@ -609,28 +668,30 @@ class MergeTest(_fixtures.FixtureTest):
         sess.add(u)
         sess.commit()
         sess.close()
-        
+
         u2 = User(id=7, name=None, address=None)
         u3 = sess.merge(u2)
         assert u3.name is None
         assert u3.address is None
-        
+
         sess.close()
-        
+
         a1 = Address(id=1, user=None)
         a2 = sess.merge(a1)
         assert a2.user is None
-        
-    @testing.resolve_artifact_names
+
     def test_transient_no_load(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
 
         sess = create_session()
         u = User()
         assert_raises_message(sa.exc.InvalidRequestError, "load=False option does not support", sess.merge, u, load=False)
 
-    @testing.resolve_artifact_names
     def test_dont_load_deprecated(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
 
         sess = create_session()
@@ -648,9 +709,14 @@ class MergeTest(_fixtures.FixtureTest):
             assert u1 not in sess.dirty
         go()
 
-    @testing.resolve_artifact_names
     def test_no_load_with_backrefs(self):
         """load=False populates relationships in both directions without requiring a load"""
+
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(mapper(Address, addresses), backref='user')
         })
@@ -679,7 +745,6 @@ class MergeTest(_fixtures.FixtureTest):
         eq_(u.addresses[1].user, User(id=7, name='fred'))
 
 
-    @testing.resolve_artifact_names
     def test_dontload_with_eager(self):
         """
 
@@ -693,6 +758,12 @@ class MergeTest(_fixtures.FixtureTest):
         'dirty'.
 
         """
+
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(mapper(Address, addresses))
         })
@@ -716,13 +787,15 @@ class MergeTest(_fixtures.FixtureTest):
             sess3.flush()
         self.assert_sql_count(testing.db, go, 0)
 
-    @testing.resolve_artifact_names
     def test_no_load_disallows_dirty(self):
         """load=False doesnt support 'dirty' objects right now
 
         (see test_no_load_with_eager()). Therefore lets assert it.
 
         """
+
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         sess = create_session()
         u = User()
@@ -751,8 +824,12 @@ class MergeTest(_fixtures.FixtureTest):
         self.assert_sql_count(testing.db, go, 0)
 
 
-    @testing.resolve_artifact_names
     def test_no_load_sets_backrefs(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(mapper(Address, addresses),backref='user')})
 
@@ -776,7 +853,6 @@ class MergeTest(_fixtures.FixtureTest):
             assert u2.addresses[0].user is u2
         self.assert_sql_count(testing.db, go, 0)
 
-    @testing.resolve_artifact_names
     def test_no_load_preserves_parents(self):
         """Merge with load=False does not trigger a 'delete-orphan' operation.
 
@@ -789,6 +865,12 @@ class MergeTest(_fixtures.FixtureTest):
         mapper._is_orphan().)
 
         """
+
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(mapper(Address, addresses),
                                  backref='user', cascade="all, delete-orphan")})
@@ -844,8 +926,9 @@ class MergeTest(_fixtures.FixtureTest):
         except sa.exc.InvalidRequestError, e:
             assert "load=False option does not support" in str(e)
 
-    @testing.resolve_artifact_names
     def test_synonym_comparable(self):
+        users = self.tables.users
+
         class User(object):
 
            class Comparator(PropComparator):
@@ -863,7 +946,7 @@ class MergeTest(_fixtures.FixtureTest):
             'uid':synonym('id'),
             'foobar':comparable_property(User.Comparator,User.value),
         })
-        
+
         sess = create_session()
         u = User()
         u.name = 'ed'
@@ -872,11 +955,16 @@ class MergeTest(_fixtures.FixtureTest):
         sess.expunge(u)
         sess.merge(u)
 
-    @testing.resolve_artifact_names
     def test_cascade_doesnt_blowaway_manytoone(self):
         """a merge test that was fixed by [ticket:1202]"""
-        
-        s = create_session(autoflush=True)
+
+        User, Address, addresses, users = (self.classes.User,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users)
+
+
+        s = create_session(autoflush=True, autocommit=False)
         mapper(User, users, properties={
             'addresses':relationship(mapper(Address, addresses),backref='user')})
 
@@ -889,10 +977,14 @@ class MergeTest(_fixtures.FixtureTest):
         eq_(after_id, other_id)
         eq_(before_id, after_id)
         eq_(a1.user, a2.user)
-            
-    @testing.resolve_artifact_names
+
     def test_cascades_dont_autoflush(self):
-        sess = create_session(autoflush=True)
+        User, Address, addresses, users = (self.classes.User,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users)
+
+        sess = create_session(autoflush=True, autocommit=False)
         m = mapper(User, users, properties={
             'addresses':relationship(mapper(Address, addresses),backref='user')})
         user = User(id=8, name='fred', addresses=[Address(email_address='user')])
@@ -901,8 +993,12 @@ class MergeTest(_fixtures.FixtureTest):
         sess.flush()
         assert merged_user not in sess.new
 
-    @testing.resolve_artifact_names
     def test_cascades_dont_autoflush_2(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address,
                         backref='user',
@@ -927,10 +1023,12 @@ class MergeTest(_fixtures.FixtureTest):
         assert sess.autoflush
         sess.commit()
 
-    @testing.resolve_artifact_names
     def test_dont_expire_pending(self):
         """test that pending instances aren't expired during a merge."""
-        
+
+        users, User = self.tables.users, self.classes.User
+
+
         mapper(User, users)
         u = User(id=7)
         sess = create_session(autoflush=True, autocommit=False)
@@ -939,40 +1037,42 @@ class MergeTest(_fixtures.FixtureTest):
         def go():
             eq_(u.name, None)
         self.assert_sql_count(testing.db, go, 0)
-    
-    @testing.resolve_artifact_names
+
     def test_option_state(self):
         """test that the merged takes on the MapperOption characteristics
         of that which is merged.
-        
+
         """
+
+        users, User = self.tables.users, self.classes.User
+
         class Option(MapperOption):
             propagate_to_loaders = True
-            
+
         opt1, opt2 = Option(), Option()
 
         sess = sessionmaker()()
-        
+
         umapper = mapper(User, users)
-        
+
         sess.add_all([
             User(id=1, name='u1'),
             User(id=2, name='u2'),
         ])
         sess.commit()
-        
+
         sess2 = sessionmaker()()
         s2_users = sess2.query(User).options(opt2).all()
-        
+
         # test 1.  no options are replaced by merge options
         sess = sessionmaker()()
         s1_users = sess.query(User).all()
-        
+
         for u in s1_users:
             ustate = attributes.instance_state(u)
             eq_(ustate.load_path, ())
             eq_(ustate.load_options, set())
-            
+
         for u in s2_users:
             sess.merge(u)
 
@@ -980,7 +1080,7 @@ class MergeTest(_fixtures.FixtureTest):
             ustate = attributes.instance_state(u)
             eq_(ustate.load_path, (umapper, ))
             eq_(ustate.load_options, set([opt2]))
-        
+
         # test 2.  present options are replaced by merge options
         sess = sessionmaker()()
         s1_users = sess.query(User).options(opt1).all()
@@ -991,67 +1091,199 @@ class MergeTest(_fixtures.FixtureTest):
 
         for u in s2_users:
             sess.merge(u)
-            
+
         for u in s1_users:
             ustate = attributes.instance_state(u)
             eq_(ustate.load_path, (umapper, ))
             eq_(ustate.load_options, set([opt2]))
-        
 
-class MutableMergeTest(_base.MappedTest):
+
+class M2ONoUseGetLoadingTest(fixtures.MappedTest):
+    """Merge a one-to-many.  The many-to-one on the other side is set up 
+    so that use_get is False.   See if skipping the "m2o" merge
+    vs. doing it saves on SQL calls.
+    
+    """
+
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('user', metadata,
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
+            Column('name', String(50)),
+        )
+        Table('address', metadata,
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
+            Column('user_id', Integer, ForeignKey('user.id')),
+            Column('email', String(50)),
+        )
+
+    @classmethod
+    def setup_classes(cls):
+        class User(cls.Comparable):
+            pass
+        class Address(cls.Comparable):
+            pass
+
+    @classmethod
+    def setup_mappers(cls):
+        User, Address = cls.classes.User, cls.classes.Address
+        user, address = cls.tables.user, cls.tables.address
+        mapper(User, user, properties={
+            'addresses':relationship(Address, backref=
+                    backref('user', 
+                        # needlessly complex primaryjoin so that the
+                        # use_get flag is False
+                        primaryjoin=and_(
+                                user.c.id==address.c.user_id, 
+                                user.c.id==user.c.id
+                           )
+                    )
+                )
+        })
+        mapper(Address, address)
+        configure_mappers()
+        assert Address.user.property._use_get is False
+
+    @classmethod
+    def insert_data(cls):
+        User, Address = cls.classes.User, cls.classes.Address
+        s = Session()
+        s.add_all([
+            User(id=1, name='u1', addresses=[Address(id=1, email='a1'), 
+                                        Address(id=2, email='a2')])
+        ])
+        s.commit()
+
+    # "persistent" - we get at an Address that was already present.
+    # With the "skip bidirectional" check removed, the "set" emits SQL 
+    # for the "previous" version in any case,
+    # address.user_id is 1, you get a load.
+    def test_persistent_access_none(self):
+        User, Address = self.classes.User, self.classes.Address
+        s = Session()
+        def go():
+            u1 = User(id=1, 
+                addresses =[Address(id=1), Address(id=2)]
+            )
+            u2 = s.merge(u1)
+        self.assert_sql_count(testing.db, go, 2)
+
+    def test_persistent_access_one(self):
+        User, Address = self.classes.User, self.classes.Address
+        s = Session()
+        def go():
+            u1 = User(id=1, 
+                addresses =[Address(id=1), Address(id=2)]
+            )
+            u2 = s.merge(u1)
+            a1 = u2.addresses[0]
+            assert a1.user is u2
+        self.assert_sql_count(testing.db, go, 3)
+
+    def test_persistent_access_two(self):
+        User, Address = self.classes.User, self.classes.Address
+        s = Session()
+        def go():
+            u1 = User(id=1, 
+                addresses =[Address(id=1), Address(id=2)]
+            )
+            u2 = s.merge(u1)
+            a1 = u2.addresses[0]
+            assert a1.user is u2
+            a2 = u2.addresses[1]
+            assert a2.user is u2
+        self.assert_sql_count(testing.db, go, 4)
+
+    # "pending" - we get at an Address that is new- user_id should be
+    # None.  But in this case the set attribute on the forward side
+    # already sets the backref.  commenting out the "skip bidirectional"
+    # check emits SQL again for the other two Address objects already
+    # persistent.
+    def test_pending_access_one(self):
+        User, Address = self.classes.User, self.classes.Address
+        s = Session()
+        def go():
+            u1 = User(id=1, 
+                addresses =[Address(id=1), Address(id=2), 
+                                Address(id=3, email='a3')]
+            )
+            u2 = s.merge(u1)
+            a3 = u2.addresses[2]
+            assert a3.user is u2
+        self.assert_sql_count(testing.db, go, 3)
+
+    def test_pending_access_two(self):
+        User, Address = self.classes.User, self.classes.Address
+        s = Session()
+        def go():
+            u1 = User(id=1, 
+                addresses =[Address(id=1), Address(id=2), 
+                                Address(id=3, email='a3')]
+            )
+            u2 = s.merge(u1)
+            a3 = u2.addresses[2]
+            assert a3.user is u2
+            a2 = u2.addresses[1]
+            assert a2.user is u2
+        self.assert_sql_count(testing.db, go, 5)
+
+class MutableMergeTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table("data", metadata, 
             Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('data', PickleType(comparator=operator.eq))
         )
-    
+
     @classmethod
     def setup_classes(cls):
-        class Data(_base.ComparableEntity):
+        class Data(cls.Basic):
             pass
-    
-    @testing.resolve_artifact_names
+
     def test_list(self):
+        Data, data = self.classes.Data, self.tables.data
+
         mapper(Data, data)
         sess = sessionmaker()()
         d = Data(data=["this", "is", "a", "list"])
-        
+
         sess.add(d)
         sess.commit()
-        
+
         d2 = Data(id=d.id, data=["this", "is", "another", "list"])
         d3 = sess.merge(d2)
         eq_(d3.data, ["this", "is", "another", "list"])
-        
-        
-        
-class CompositeNullPksTest(_base.MappedTest):
+
+
+
+class CompositeNullPksTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table("data", metadata, 
             Column('pk1', String(10), primary_key=True),
             Column('pk2', String(10), primary_key=True),
         )
-    
+
     @classmethod
     def setup_classes(cls):
-        class Data(_base.ComparableEntity):
+        class Data(cls.Basic):
             pass
-    
-    @testing.resolve_artifact_names
+
     def test_merge_allow_partial(self):
+        Data, data = self.classes.Data, self.tables.data
+
         mapper(Data, data)
         sess = sessionmaker()()
-        
+
         d1 = Data(pk1="someval", pk2=None)
-        
+
         def go():
             return sess.merge(d1)
         self.assert_sql_count(testing.db, go, 1)
 
-    @testing.resolve_artifact_names
     def test_merge_disallow_partial(self):
+        Data, data = self.classes.Data, self.tables.data
+
         mapper(Data, data, allow_partial_pks=False)
         sess = sessionmaker()()
 
@@ -1060,5 +1292,5 @@ class CompositeNullPksTest(_base.MappedTest):
         def go():
             return sess.merge(d1)
         self.assert_sql_count(testing.db, go, 0)
-    
+
 
