@@ -2,7 +2,7 @@ from test.lib.testing import eq_, assert_raises, assert_raises_message
 import operator
 from sqlalchemy import MetaData, null, exists, text, union, literal, \
     literal_column, func, between, Unicode, desc, and_, bindparam, \
-    select, distinct, or_
+    select, distinct, or_, collate
 from sqlalchemy import exc as sa_exc, util
 from sqlalchemy.sql import compiler, table, column
 from sqlalchemy.sql import expression
@@ -432,6 +432,13 @@ class InvalidGenerationsTest(QueryTest, AssertsCompiledSQL):
         q = s.query(User)
         assert_raises(sa_exc.InvalidRequestError, q.add_column, object())
 
+    def test_invalid_column_tuple(self):
+        User = self.classes.User
+
+        s = create_session()
+        q = s.query(User)
+        assert_raises(sa_exc.InvalidRequestError, q.add_column, (1, 1))
+
     def test_distinct(self):
         """test that a distinct() call is not valid before 'clauseelement' conditions."""
 
@@ -718,6 +725,15 @@ class OperatorTest(QueryTest, AssertsCompiledSQL):
         self._test(User.id.between('a', 'b'),
                    "users.id BETWEEN :id_1 AND :id_2")
 
+    def test_collate(self):
+        User = self.classes.User
+
+        self._test(collate(User.id, 'binary'),
+                   "users.id COLLATE binary")
+
+        self._test(User.id.collate('binary'),
+                   "users.id COLLATE binary")
+
     def test_selfref_between(self):
         User = self.classes.User
 
@@ -770,6 +786,16 @@ class ExpressionTest(QueryTest, AssertsCompiledSQL):
             "ON users.id = addresses.user_id) AS anon_1"
         )
 
+    def test_aliased_sql_construct_raises_adapt_on_names(self):
+        User, Address = self.classes.User, self.classes.Address
+
+        j = join(User, Address)
+        assert_raises_message(
+            sa_exc.ArgumentError,
+            "adapt_on_names only applies to ORM elements",
+            aliased, j, adapt_on_names=True
+        )
+
     def test_scalar_subquery_compile_whereclause(self):
         User = self.classes.User
         Address = self.classes.Address
@@ -826,7 +852,7 @@ class ExpressionTest(QueryTest, AssertsCompiledSQL):
                             'IN (SELECT users.id FROM users WHERE '
                             'users.id = :id_1)')
 
-
+    @testing.fails_on('mssql', "mssql doesn't allow col = <subquery>, sqla deprecated workaround")
     def test_param_transfer(self):
         User = self.classes.User
 
@@ -1779,8 +1805,9 @@ class SynonymTest(QueryTest):
         mapper(User, users, properties={
             'name_syn':synonym('name'),
             'addresses':relationship(Address),
-            'orders':relationship(Order, backref='user'), # o2m, m2o
-            'orders_syn':synonym('orders')
+            'orders':relationship(Order, backref='user', order_by=orders.c.id), # o2m, m2o
+            'orders_syn':synonym('orders'),
+            'orders_syn_2':synonym('orders_syn')
         })
         mapper(Address, addresses)
         mapper(Order, orders, properties={
@@ -1793,22 +1820,70 @@ class SynonymTest(QueryTest):
         })
         mapper(Keyword, keywords)
 
-    @testing.fails_if(lambda: True, "0.7 regression, may not support "
-                                "synonyms for relationship")
+    def test_options(self):
+        User, Order = self.classes.User, self.classes.Order
+
+        s = create_session()
+        def go():
+            result = s.query(User).filter_by(name='jack').\
+                        options(joinedload(User.orders_syn)).all()
+            eq_(result, [
+                User(id=7, name='jack', orders=[
+                    Order(description=u'order 1'), 
+                    Order(description=u'order 3'), 
+                    Order(description=u'order 5')
+                ])
+            ])
+        self.assert_sql_count(testing.db, go, 1)
+
+    def test_options_syn_of_syn(self):
+        User, Order = self.classes.User, self.classes.Order
+
+        s = create_session()
+        def go():
+            result = s.query(User).filter_by(name='jack').\
+                        options(joinedload(User.orders_syn_2)).all()
+            eq_(result, [
+                User(id=7, name='jack', orders=[
+                    Order(description=u'order 1'), 
+                    Order(description=u'order 3'), 
+                    Order(description=u'order 5')
+                ])
+            ])
+        self.assert_sql_count(testing.db, go, 1)
+
+    def test_options_syn_of_syn_string(self):
+        User, Order = self.classes.User, self.classes.Order
+
+        s = create_session()
+        def go():
+            result = s.query(User).filter_by(name='jack').\
+                        options(joinedload('orders_syn_2')).all()
+            eq_(result, [
+                User(id=7, name='jack', orders=[
+                    Order(description=u'order 1'), 
+                    Order(description=u'order 3'), 
+                    Order(description=u'order 5')
+                ])
+            ])
+        self.assert_sql_count(testing.db, go, 1)
+
     def test_joins(self):
-        User = self.classes.User
+        User, Order = self.classes.User, self.classes.Order
 
         for j in (
             ['orders', 'items'],
             ['orders_syn', 'items'],
+            [User.orders_syn, Order.items],
+            ['orders_syn_2', 'items'],
+            [User.orders_syn_2, 'items'],
             ['orders', 'items_syn'],
             ['orders_syn', 'items_syn'],
+            ['orders_syn_2', 'items_syn'],
         ):
             result = create_session().query(User).join(*j).filter_by(id=3).all()
             assert [User(id=7, name='jack'), User(id=9, name='fred')] == result
 
-    @testing.fails_if(lambda: True, "0.7 regression, may not support "
-                                "synonyms for relationship")
     def test_with_parent(self):
         Order, User = self.classes.Order, self.classes.User
 
@@ -1816,7 +1891,9 @@ class SynonymTest(QueryTest):
             ('name', 'orders'),
             ('name_syn', 'orders'),
             ('name', 'orders_syn'),
+            ('name', 'orders_syn_2'),
             ('name_syn', 'orders_syn'),
+            ('name_syn', 'orders_syn_2'),
         ):
             sess = create_session()
             q = sess.query(User)
@@ -1824,7 +1901,8 @@ class SynonymTest(QueryTest):
             u1 = q.filter_by(**{nameprop:'jack'}).one()
 
             o = sess.query(Order).with_parent(u1, property=orderprop).all()
-            assert [Order(description="order 1"), Order(description="order 3"), Order(description="order 5")] == o
+            assert [Order(description="order 1"), 
+                    Order(description="order 3"), Order(description="order 5")] == o
 
 
 class ImmediateTest(_fixtures.FixtureTest):
