@@ -1,14 +1,14 @@
 
-from sqlalchemy.test.testing import eq_, assert_raises, assert_raises_message
+from test.lib.testing import eq_, assert_raises, assert_raises_message
 from sqlalchemy import *
 from sqlalchemy.orm import attributes
-from sqlalchemy import exc as sa_exc
+from sqlalchemy import exc as sa_exc, event
 from sqlalchemy.orm import exc as orm_exc
 from sqlalchemy.orm import *
-from sqlalchemy.test.util import gc_collect
-from sqlalchemy.test import testing
-from test.orm import _base
-from test.orm._fixtures import FixtureTest, User, Address, users, addresses
+from test.lib.util import gc_collect
+from test.lib import testing
+from test.lib import fixtures
+from test.orm._fixtures import FixtureTest
 
 class TransactionTest(FixtureTest):
     run_setup_mappers = 'once'
@@ -17,6 +17,8 @@ class TransactionTest(FixtureTest):
 
     @classmethod
     def setup_mappers(cls):
+        User, Address = cls.classes.User, cls.classes.Address
+        users, addresses = cls.tables.users, cls.tables.addresses
         mapper(User, users, properties={
             'addresses':relationship(Address, backref='user',
                                  cascade="all, delete-orphan", order_by=addresses.c.id),
@@ -24,11 +26,11 @@ class TransactionTest(FixtureTest):
         mapper(Address, addresses)
 
 
-    
 class FixtureDataTest(TransactionTest):
     run_inserts = 'each'
-    
+
     def test_attrs_on_rollback(self):
+        User = self.classes.User
         sess = self.session()
         u1 = sess.query(User).get(7)
         u1.name = 'ed'
@@ -36,6 +38,7 @@ class FixtureDataTest(TransactionTest):
         eq_(u1.name, 'jack')
 
     def test_commit_persistent(self):
+        User = self.classes.User
         sess = self.session()
         u1 = sess.query(User).get(7)
         u1.name = 'ed'
@@ -44,6 +47,7 @@ class FixtureDataTest(TransactionTest):
         eq_(u1.name, 'ed')
 
     def test_concurrent_commit_persistent(self):
+        User = self.classes.User
         s1 = self.session()
         u1 = s1.query(User).get(7)
         u1.name = 'ed'
@@ -60,6 +64,7 @@ class FixtureDataTest(TransactionTest):
 class AutoExpireTest(TransactionTest):
 
     def test_expunge_pending_on_rollback(self):
+        User = self.classes.User
         sess = self.session()
         u2= User(name='newuser')
         sess.add(u2)
@@ -68,6 +73,7 @@ class AutoExpireTest(TransactionTest):
         assert u2 not in sess
 
     def test_trans_pending_cleared_on_commit(self):
+        User = self.classes.User
         sess = self.session()
         u2= User(name='newuser')
         sess.add(u2)
@@ -81,6 +87,7 @@ class AutoExpireTest(TransactionTest):
         assert u2 in sess
 
     def test_update_deleted_on_rollback(self):
+        User = self.classes.User
         s = self.session()
         u1 = User(name='ed')
         s.add(u1)
@@ -94,13 +101,15 @@ class AutoExpireTest(TransactionTest):
         s.rollback()
         assert u1 in s
         assert u1 not in s.deleted
-    
+
     def test_gced_delete_on_rollback(self):
+        User, users = self.classes.User, self.tables.users
+
         s = self.session()
         u1 = User(name='ed')
         s.add(u1)
         s.commit()
-        
+
         s.delete(u1)
         u1_state = attributes.instance_state(u1)
         assert u1_state in s.identity_map.all_states()
@@ -111,7 +120,7 @@ class AutoExpireTest(TransactionTest):
         del u1
         gc_collect()
         assert u1_state.obj() is None
-        
+
         s.rollback()
         assert u1_state in s.identity_map.all_states()
         u1 = s.query(User).filter_by(name='ed').one()
@@ -121,8 +130,9 @@ class AutoExpireTest(TransactionTest):
         s.flush()
         assert s.scalar(users.count()) == 0
         s.commit()
-        
+
     def test_trans_deleted_cleared_on_rollback(self):
+        User = self.classes.User
         s = self.session()
         u1 = User(name='ed')
         s.add(u1)
@@ -135,6 +145,8 @@ class AutoExpireTest(TransactionTest):
         assert u1 not in s
 
     def test_update_deleted_on_rollback_cascade(self):
+        User, Address = self.classes.User, self.classes.Address
+
         s = self.session()
         u1 = User(name='ed', addresses=[Address(email_address='foo')])
         s.add(u1)
@@ -149,6 +161,8 @@ class AutoExpireTest(TransactionTest):
         assert u1.addresses[0] not in s.deleted
 
     def test_update_deleted_on_rollback_orphan(self):
+        User, Address = self.classes.User, self.classes.Address
+
         s = self.session()
         u1 = User(name='ed', addresses=[Address(email_address='foo')])
         s.add(u1)
@@ -164,6 +178,7 @@ class AutoExpireTest(TransactionTest):
         assert u1.addresses == [a1]
 
     def test_commit_pending(self):
+        User = self.classes.User
         sess = self.session()
         u1 = User(name='newuser')
         sess.add(u1)
@@ -173,6 +188,7 @@ class AutoExpireTest(TransactionTest):
 
 
     def test_concurrent_commit_pending(self):
+        User = self.classes.User
         s1 = self.session()
         u1 = User(name='edward')
         s1.add(u1)
@@ -189,18 +205,20 @@ class TwoPhaseTest(TransactionTest):
 
     @testing.requires.two_phase_transactions
     def test_rollback_on_prepare(self):
+        User = self.classes.User
         s = self.session(twophase=True)
-    
+
         u = User(name='ed')
         s.add(u)
         s.prepare()
         s.rollback()
-        
+
         assert u not in s
-        
+
 class RollbackRecoverTest(TransactionTest):
 
     def test_pk_violation(self):
+        User, Address = self.classes.User, self.classes.Address
         s = self.session()
         a1 = Address(email_address='foo')
         u1 = User(id=1, name='ed', addresses=[a1])
@@ -232,6 +250,7 @@ class RollbackRecoverTest(TransactionTest):
 
     @testing.requires.savepoints
     def test_pk_violation_with_savepoint(self):
+        User, Address = self.classes.User, self.classes.Address
         s = self.session()
         a1 = Address(email_address='foo')
         u1 = User(id=1, name='ed', addresses=[a1])
@@ -261,6 +280,7 @@ class SavepointTest(TransactionTest):
 
     @testing.requires.savepoints
     def test_savepoint_rollback(self):
+        User = self.classes.User
         s = self.session()
         u1 = User(name='ed')
         u2 = User(name='jack')
@@ -284,6 +304,7 @@ class SavepointTest(TransactionTest):
 
     @testing.requires.savepoints
     def test_savepoint_delete(self):
+        User = self.classes.User
         s = self.session()
         u1 = User(name='ed')
         s.add(u1)
@@ -297,6 +318,7 @@ class SavepointTest(TransactionTest):
 
     @testing.requires.savepoints
     def test_savepoint_commit(self):
+        User = self.classes.User
         s = self.session()
         u1 = User(name='ed')
         u2 = User(name='jack')
@@ -321,6 +343,7 @@ class SavepointTest(TransactionTest):
 
     @testing.requires.savepoints
     def test_savepoint_rollback_collections(self):
+        User, Address = self.classes.User, self.classes.Address
         s = self.session()
         u1 = User(name='ed', addresses=[Address(email_address='foo')])
         s.add(u1)
@@ -352,6 +375,7 @@ class SavepointTest(TransactionTest):
 
     @testing.requires.savepoints
     def test_savepoint_commit_collections(self):
+        User, Address = self.classes.User, self.classes.Address
         s = self.session()
         u1 = User(name='ed', addresses=[Address(email_address='foo')])
         s.add(u1)
@@ -385,6 +409,7 @@ class SavepointTest(TransactionTest):
 
     @testing.requires.savepoints
     def test_expunge_pending_on_rollback(self):
+        User = self.classes.User
         sess = self.session()
 
         sess.begin_nested()
@@ -396,6 +421,7 @@ class SavepointTest(TransactionTest):
 
     @testing.requires.savepoints
     def test_update_deleted_on_rollback(self):
+        User = self.classes.User
         s = self.session()
         u1 = User(name='ed')
         s.add(u1)
@@ -411,18 +437,22 @@ class SavepointTest(TransactionTest):
 
 class AccountingFlagsTest(TransactionTest):
     def test_no_expire_on_commit(self):
+        User, users = self.classes.User, self.tables.users
+
         sess = sessionmaker(expire_on_commit=False)()
         u1 = User(name='ed')
         sess.add(u1)
         sess.commit()
 
         testing.db.execute(users.update(users.c.name=='ed').values(name='edward'))
-        
+
         assert u1.name == 'ed'
         sess.expire_all()
         assert u1.name == 'edward'
 
     def test_rollback_no_accounting(self):
+        User, users = self.classes.User, self.tables.users
+
         sess = sessionmaker(_enable_transaction_accounting=False)()
         u1 = User(name='ed')
         sess.add(u1)
@@ -430,7 +460,7 @@ class AccountingFlagsTest(TransactionTest):
 
         u1.name = 'edwardo'
         sess.rollback()
-        
+
         testing.db.execute(users.update(users.c.name=='ed').values(name='edward'))
 
         assert u1.name == 'edwardo'
@@ -438,6 +468,8 @@ class AccountingFlagsTest(TransactionTest):
         assert u1.name == 'edward'
 
     def test_commit_no_accounting(self):
+        User, users = self.classes.User, self.tables.users
+
         sess = sessionmaker(_enable_transaction_accounting=False)()
         u1 = User(name='ed')
         sess.add(u1)
@@ -450,41 +482,45 @@ class AccountingFlagsTest(TransactionTest):
 
         assert u1.name == 'edwardo'
         sess.commit()
-        
+
         assert testing.db.execute(select([users.c.name])).fetchall() == [('edwardo',)]
         assert u1.name == 'edwardo'
 
         sess.delete(u1)
         sess.commit()
-        
+
     def test_preflush_no_accounting(self):
-        sess = sessionmaker(_enable_transaction_accounting=False, autocommit=True)()
+        User, users = self.classes.User, self.tables.users
+
+        sess = Session(_enable_transaction_accounting=False, 
+                        autocommit=True, autoflush=False)
         u1 = User(name='ed')
         sess.add(u1)
         sess.flush()
-        
+
         sess.begin()
         u1.name = 'edwardo'
         u2 = User(name="some other user")
         sess.add(u2)
-        
+
         sess.rollback()
 
         sess.begin()
         assert testing.db.execute(select([users.c.name])).fetchall() == [('ed',)]
-        
-    
+
+
 class AutoCommitTest(TransactionTest):
     def test_begin_nested_requires_trans(self):
         sess = create_session(autocommit=True)
         assert_raises(sa_exc.InvalidRequestError, sess.begin_nested)
 
     def test_begin_preflush(self):
+        User = self.classes.User
         sess = create_session(autocommit=True)
 
         u1 = User(name='ed')
         sess.add(u1)
-        
+
         sess.begin()
         u2 = User(name='some other user')
         sess.add(u2)
@@ -493,7 +529,88 @@ class AutoCommitTest(TransactionTest):
         assert u1 in sess
         assert sess.query(User).filter_by(name='ed').one() is u1
 
-class NaturalPKRollbackTest(_base.MappedTest):
+    def test_accounting_commit_fails_add(self):
+        User = self.classes.User
+        sess = create_session(autocommit=True)
+
+        fail = False
+        def fail_fn(*arg, **kw):
+            if fail:
+                raise Exception("commit fails")
+
+        event.listen(sess, "after_flush_postexec", fail_fn)
+        u1 = User(name='ed')
+        sess.add(u1)
+
+        fail = True
+        assert_raises(
+            Exception,
+            sess.flush
+        )
+        fail = False
+
+        assert u1 not in sess
+        u1new = User(id=2, name='fred')
+        sess.add(u1new)
+        sess.add(u1)
+        sess.flush()
+        assert u1 in sess
+        eq_(
+            sess.query(User.name).order_by(User.name).all(),
+            [('ed', ), ('fred',)]
+        )
+
+    def test_accounting_commit_fails_delete(self):
+        User = self.classes.User
+        sess = create_session(autocommit=True)
+
+        fail = False
+        def fail_fn(*arg, **kw):
+            if fail:
+                raise Exception("commit fails")
+
+        event.listen(sess, "after_flush_postexec", fail_fn)
+        u1 = User(name='ed')
+        sess.add(u1)
+        sess.flush()
+
+        sess.delete(u1)
+        fail = True
+        assert_raises(
+            Exception,
+            sess.flush
+        )
+        fail = False
+
+        assert u1 in sess
+        assert u1 not in sess.deleted
+        sess.delete(u1)
+        sess.flush()
+        assert u1 not in sess
+        eq_(
+            sess.query(User.name).order_by(User.name).all(),
+            []
+        )
+
+    @testing.requires.updateable_autoincrement_pks
+    def test_accounting_no_select_needed(self):
+        """test that flush accounting works on non-expired instances
+        when autocommit=True/expire_on_commit=True."""
+
+        User = self.classes.User
+        sess = create_session(autocommit=True, expire_on_commit=True)
+
+        u1 = User(id=1, name='ed')
+        sess.add(u1)
+        sess.flush()
+
+        u1.id = 3
+        u1.name = 'fred'
+        self.assert_sql_count(testing.db, sess.flush, 1)
+        assert 'id' not in u1.__dict__
+        eq_(u1.id, 3)
+
+class NaturalPKRollbackTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('users', metadata,
@@ -502,11 +619,12 @@ class NaturalPKRollbackTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class User(_base.ComparableEntity):
+        class User(cls.Comparable):
             pass
 
-    @testing.resolve_artifact_names
     def test_rollback_recover(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
 
         session = sessionmaker()()
@@ -534,7 +652,7 @@ class NaturalPKRollbackTest(_base.MappedTest):
 
         session.rollback()
 
-        
-        
+
+
 
 
