@@ -1,17 +1,18 @@
-from sqlalchemy.test.testing import eq_, assert_raises, assert_raises_message
+from test.lib.testing import eq_, assert_raises, assert_raises_message
 import datetime
-from sqlalchemy import Sequence, Column, func
 from sqlalchemy.schema import CreateSequence, DropSequence
-from sqlalchemy.sql import select, text
+from sqlalchemy.sql import select, text, literal_column
 import sqlalchemy as sa
-from sqlalchemy.test import testing, engines
-from sqlalchemy import MetaData, Integer, String, ForeignKey, Boolean, exc
-from sqlalchemy.test.schema import Table
-from sqlalchemy.test.testing import eq_
-from test.sql import _base
+from test.lib import testing, engines
+from sqlalchemy import MetaData, Integer, String, ForeignKey, Boolean, exc,\
+                Sequence, func, literal, Unicode
+from sqlalchemy.types import TypeDecorator, TypeEngine
+from test.lib.schema import Table, Column
+from test.lib.testing import eq_
+from sqlalchemy.dialects import sqlite
+from test.lib import fixtures
 
-
-class DefaultTest(testing.TestBase):
+class DefaultTest(fixtures.TestBase):
 
     @classmethod
     def setup_class(cls):
@@ -67,7 +68,7 @@ class DefaultTest(testing.TestBase):
                 def2 = sa.text("getdate()")
             else:
                 def2 = sa.text("current_date")
-            ts = db.func.current_date().scalar()
+            ts = db.scalar(func.current_date())
         else:
             f = len('abcdef')
             f2 = len('abcdefghijk')
@@ -302,8 +303,34 @@ class DefaultTest(testing.TestBase):
              (53, 'imthedefault', f, ts, ts, ctexec, True, False,
               12, today, 'py')])
 
+    def test_no_embed_in_sql(self):
+        """Using a DefaultGenerator, Sequence, DefaultClause
+        in the columns, where clause of a select, or in the values 
+        clause of insert, update, raises an informative error"""
+
+        for const in (
+            sa.Sequence('y'),
+            sa.ColumnDefault('y'),
+            sa.DefaultClause('y')
+        ):
+            assert_raises_message(
+                sa.exc.ArgumentError,
+                "SQL expression object or string expected.",
+                t.select, [const]
+            )
+            assert_raises_message(
+                sa.exc.InvalidRequestError,
+                "cannot be used directly as a column expression.",
+                str, t.insert().values(col4=const)
+            )
+            assert_raises_message(
+                sa.exc.InvalidRequestError,
+                "cannot be used directly as a column expression.",
+                str, t.update().values(col4=const)
+            )
+
     def test_missing_many_param(self):
-        assert_raises_message(exc.InvalidRequestError, 
+        assert_raises_message(exc.StatementError, 
             "A value is required for bind parameter 'col7', in parameter group 1",
             t.insert().execute,
             {'col4':7, 'col7':12, 'col8':19},
@@ -367,7 +394,7 @@ class DefaultTest(testing.TestBase):
         eq_(55, l['col3'])
 
 
-class PKDefaultTest(_base.TablesTest):
+class PKDefaultTest(fixtures.TablesTest):
     __requires__ = ('subqueries',)
 
     @classmethod
@@ -387,8 +414,9 @@ class PKDefaultTest(_base.TablesTest):
     def test_regular(self):
         self._test(False)
 
-    @testing.resolve_artifact_names
     def _test(self, returning):
+        t2, t1 = self.tables.t2, self.tables.t1
+
         if not returning and not testing.db.dialect.implicit_returning:
             engine = testing.db
         else:
@@ -401,7 +429,7 @@ class PKDefaultTest(_base.TablesTest):
         r = engine.execute(t1.insert(), data='there')
         eq_([2], r.inserted_primary_key)
 
-class PKIncrementTest(_base.TablesTest):
+class PKIncrementTest(fixtures.TablesTest):
     run_define_tables = 'each'
 
     @classmethod
@@ -414,8 +442,9 @@ class PKIncrementTest(_base.TablesTest):
 
     # TODO: add coverage for increment on a secondary column in a key
     @testing.fails_on('firebird', 'Data type unknown')
-    @testing.resolve_artifact_names
     def _test_autoincrement(self, bind):
+        aitable = self.tables.aitable
+
         ids = set()
         rs = bind.execute(aitable.insert(), int1=1)
         last = rs.inserted_primary_key[0]
@@ -446,11 +475,9 @@ class PKIncrementTest(_base.TablesTest):
         eq_(list(bind.execute(aitable.select().order_by(aitable.c.id))),
             [(1, 1, None), (2, None, 'row 2'), (3, 3, 'row 3'), (4, 4, None)])
 
-    @testing.resolve_artifact_names
     def test_autoincrement_autocommit(self):
         self._test_autoincrement(testing.db)
 
-    @testing.resolve_artifact_names
     def test_autoincrement_transaction(self):
         con = testing.db.connect()
         tx = con.begin()
@@ -469,7 +496,7 @@ class PKIncrementTest(_base.TablesTest):
             con.close()
 
 
-class EmptyInsertTest(testing.TestBase):
+class EmptyInsertTest(fixtures.TestBase):
     @testing.exclude('sqlite', '<', (3, 3, 8), 'no empty insert support')
     @testing.fails_on('oracle', 'FIXME: unknown')
     def test_empty_insert(self):
@@ -485,7 +512,7 @@ class EmptyInsertTest(testing.TestBase):
         finally:
             metadata.drop_all()
 
-class AutoIncrementTest(_base.TablesTest):
+class AutoIncrementTest(fixtures.TablesTest):
     __requires__ = ('identity',)
     run_define_tables = 'each'
 
@@ -515,6 +542,40 @@ class AutoIncrementTest(_base.TablesTest):
         id_ = r.inserted_primary_key[0]
         nodes.insert().execute(data='bar', parent_id=id_)
 
+    def test_autoinc_detection_no_affinity(self):
+        class MyType(TypeDecorator):
+            impl = TypeEngine
+
+        assert MyType()._type_affinity is None
+        t = Table('x', MetaData(),
+            Column('id', MyType(), primary_key=True)
+        )
+        assert t._autoincrement_column is None
+
+    def test_autoincrement_ignore_fk(self):
+        m = MetaData()
+        y = Table('y', m,
+            Column('id', Integer(), primary_key=True)
+        )
+        x = Table('x', m,
+            Column('id', Integer(), 
+                ForeignKey('y.id'), 
+                autoincrement="ignore_fk", primary_key=True)
+        )
+        assert x._autoincrement_column is x.c.id
+
+    def test_autoincrement_fk_disqualifies(self):
+        m = MetaData()
+        y = Table('y', m,
+            Column('id', Integer(), primary_key=True)
+        )
+        x = Table('x', m,
+            Column('id', Integer(), 
+                ForeignKey('y.id'), 
+                primary_key=True)
+        )
+        assert x._autoincrement_column is None
+
     @testing.fails_on('sqlite', 'FIXME: unknown')
     def test_non_autoincrement(self):
         # sqlite INT primary keys can be non-unique! (only for ints)
@@ -530,68 +591,175 @@ class AutoIncrementTest(_base.TablesTest):
             nonai.insert().execute(data='row 1')
             nonai.insert().execute(data='row 2')
             assert False
-        except sa.exc.SQLError, e:
+        except sa.exc.DBAPIError, e:
             assert True
 
         nonai.insert().execute(id=1, data='row 1')
 
+class SequenceDDLTest(fixtures.TestBase, testing.AssertsCompiledSQL):
+    __dialect__ = 'default'
 
-class SequenceTest(testing.TestBase, testing.AssertsCompiledSQL):
-
-    @classmethod
-    @testing.requires.sequences
-    def setup_class(cls):
-        global cartitems, sometable, metadata
-        metadata = MetaData(testing.db)
-        cartitems = Table("cartitems", metadata,
-            Column("cart_id", Integer, Sequence('cart_id_seq'), primary_key=True),
-            Column("description", String(40)),
-            Column("createdate", sa.DateTime())
-        )
-        sometable = Table( 'Manager', metadata,
-               Column('obj_id', Integer, Sequence('obj_id_seq'), ),
-               Column('name', String(128)),
-               Column('id', Integer, Sequence('Manager_id_seq', optional=True),
-                      primary_key=True),
-           )
-
-        metadata.create_all()
-
-
-    def test_compile(self):
+    def test_create_drop_ddl(self):
         self.assert_compile(
             CreateSequence(Sequence('foo_seq')),
             "CREATE SEQUENCE foo_seq",
-            use_default_dialect=True,
         )
 
         self.assert_compile(
             CreateSequence(Sequence('foo_seq', start=5)),
             "CREATE SEQUENCE foo_seq START WITH 5",
-            use_default_dialect=True,
         )
 
         self.assert_compile(
             CreateSequence(Sequence('foo_seq', increment=2)),
             "CREATE SEQUENCE foo_seq INCREMENT BY 2",
-            use_default_dialect=True,
         )
 
         self.assert_compile(
             CreateSequence(Sequence('foo_seq', increment=2, start=5)),
             "CREATE SEQUENCE foo_seq INCREMENT BY 2 START WITH 5",
-            use_default_dialect=True,
         )
 
         self.assert_compile(
             DropSequence(Sequence('foo_seq')),
             "DROP SEQUENCE foo_seq",
-            use_default_dialect=True,
         )
 
+class SequenceExecTest(fixtures.TestBase):
+    __requires__ = ('sequences',)
+
+    @classmethod
+    def setup_class(cls):
+        cls.seq= Sequence("my_sequence")
+        cls.seq.create(testing.db)
+
+    @classmethod
+    def teardown_class(cls):
+        cls.seq.drop(testing.db)
+
+    def _assert_seq_result(self, ret):
+        """asserts return of next_value is an int"""
+
+        assert isinstance(ret, (int, long))
+        assert ret > 0
+
+    def test_implicit_connectionless(self):
+        s = Sequence("my_sequence", metadata=MetaData(testing.db))
+        self._assert_seq_result(s.execute())
+
+    def test_explicit(self):
+        s = Sequence("my_sequence")
+        self._assert_seq_result(s.execute(testing.db))
+
+    def test_explicit_optional(self):
+        """test dialect executes a Sequence, returns nextval, whether 
+        or not "optional" is set """
+
+        s = Sequence("my_sequence", optional=True)
+        self._assert_seq_result(s.execute(testing.db))
+
+    def test_func_implicit_connectionless_execute(self):
+        """test func.next_value().execute()/.scalar() works
+        with connectionless execution. """
+
+        s = Sequence("my_sequence", metadata=MetaData(testing.db))
+        self._assert_seq_result(s.next_value().execute().scalar())
+
+    def test_func_explicit(self):
+        s = Sequence("my_sequence")
+        self._assert_seq_result(testing.db.scalar(s.next_value()))
+
+    def test_func_implicit_connectionless_scalar(self):
+        """test func.next_value().execute()/.scalar() works. """
+
+        s = Sequence("my_sequence", metadata=MetaData(testing.db))
+        self._assert_seq_result(s.next_value().scalar())
+
+    def test_func_embedded_select(self):
+        """test can use next_value() in select column expr"""
+
+        s = Sequence("my_sequence")
+        self._assert_seq_result(
+            testing.db.scalar(select([s.next_value()]))
+        )
+
+    @testing.fails_on('oracle', "ORA-02287: sequence number not allowed here")
+    @testing.provide_metadata
+    def test_func_embedded_whereclause(self):
+        """test can use next_value() in whereclause"""
+
+        metadata = self.metadata
+        t1 = Table('t', metadata,
+            Column('x', Integer)
+        )
+        t1.create(testing.db)
+        testing.db.execute(t1.insert(), [{'x':1}, {'x':300}, {'x':301}])
+        s = Sequence("my_sequence")
+        eq_(
+            testing.db.execute(
+                t1.select().where(t1.c.x > s.next_value())
+            ).fetchall(),
+            [(300, ), (301, )]
+        )
+
+    @testing.provide_metadata
+    def test_func_embedded_valuesbase(self):
+        """test can use next_value() in values() of _ValuesBase"""
+
+        metadata = self.metadata
+        t1 = Table('t', metadata,
+            Column('x', Integer)
+        )
+        t1.create(testing.db)
+        s = Sequence("my_sequence")
+        testing.db.execute(
+            t1.insert().values(x=s.next_value())
+        )
+        self._assert_seq_result(
+            testing.db.scalar(t1.select())
+        )
+
+    @testing.provide_metadata
+    def test_inserted_pk_no_returning(self):
+        """test inserted_primary_key contains [None] when 
+        pk_col=next_value(), implicit returning is not used."""
+
+        metadata = self.metadata
+        e = engines.testing_engine(options={'implicit_returning':False})
+        s = Sequence("my_sequence")
+        metadata.bind = e
+        t1 = Table('t', metadata,
+            Column('x', Integer, primary_key=True)
+        )
+        t1.create()
+        r = e.execute(
+            t1.insert().values(x=s.next_value())
+        )
+        eq_(r.inserted_primary_key, [None])
+
+    @testing.requires.returning
+    @testing.provide_metadata
+    def test_inserted_pk_implicit_returning(self):
+        """test inserted_primary_key contains the result when 
+        pk_col=next_value(), when implicit returning is used."""
+
+        metadata = self.metadata
+        e = engines.testing_engine(options={'implicit_returning':True})
+        s = Sequence("my_sequence")
+        metadata.bind = e
+        t1 = Table('t', metadata,
+            Column('x', Integer, primary_key=True)
+        )
+        t1.create()
+        r = e.execute(
+            t1.insert().values(x=s.next_value())
+        )
+        self._assert_seq_result(r.inserted_primary_key[0])
+
+class SequenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
+    __requires__ = ('sequences',)
 
     @testing.fails_on('firebird', 'no FB support for start/increment')
-    @testing.requires.sequences
     def test_start_increment(self):
         for seq in (
                 Sequence('foo_seq'), 
@@ -609,7 +777,138 @@ class SequenceTest(testing.TestBase, testing.AssertsCompiledSQL):
             finally:
                 seq.drop(testing.db)
 
-    @testing.requires.sequences
+
+    def _has_sequence(self, name):
+        return testing.db.dialect.has_sequence(testing.db, name)
+
+    def test_nextval_render(self):
+        """test dialect renders the "nextval" construct, 
+        whether or not "optional" is set """
+
+        for s in (
+                Sequence("my_seq"), 
+                Sequence("my_seq", optional=True)):
+            assert str(s.next_value().
+                    compile(dialect=testing.db.dialect)) in (
+                "nextval('my_seq')",
+                "gen_id(my_seq, 1)",
+                "my_seq.nextval",
+            )
+
+    def test_nextval_unsupported(self):
+        """test next_value() used on non-sequence platform 
+        raises NotImplementedError."""
+
+        s = Sequence("my_seq")
+        d = sqlite.dialect()
+        assert_raises_message(
+            NotImplementedError,
+            "Dialect 'sqlite' does not support sequence increments.",
+            s.next_value().compile,
+            dialect=d
+        )
+
+    def test_checkfirst_sequence(self):
+        s = Sequence("my_sequence")
+        s.create(testing.db, checkfirst=False)
+        assert self._has_sequence('my_sequence')
+        s.create(testing.db, checkfirst=True)
+        s.drop(testing.db, checkfirst=False)
+        assert not self._has_sequence('my_sequence')
+        s.drop(testing.db, checkfirst=True)
+
+    def test_checkfirst_metadata(self):
+        m = MetaData()
+        s = Sequence("my_sequence", metadata=m)
+        m.create_all(testing.db, checkfirst=False)
+        assert self._has_sequence('my_sequence')
+        m.create_all(testing.db, checkfirst=True)
+        m.drop_all(testing.db, checkfirst=False)
+        assert not self._has_sequence('my_sequence')
+        m.drop_all(testing.db, checkfirst=True)
+
+    def test_checkfirst_table(self):
+        m = MetaData()
+        s = Sequence("my_sequence")
+        t = Table('t', m, Column('c', Integer, s, primary_key=True))
+        t.create(testing.db, checkfirst=False)
+        assert self._has_sequence('my_sequence')
+        t.create(testing.db, checkfirst=True)
+        t.drop(testing.db, checkfirst=False)
+        assert not self._has_sequence('my_sequence')
+        t.drop(testing.db, checkfirst=True)
+
+    @testing.provide_metadata
+    def test_table_overrides_metadata_create(self):
+        metadata = self.metadata
+        s1 = Sequence("s1", metadata=metadata)
+        s2 = Sequence("s2", metadata=metadata)
+        s3 = Sequence("s3")
+        t = Table('t', metadata, 
+                    Column('c', Integer, s3, primary_key=True))
+        assert s3.metadata is metadata
+
+
+        t.create(testing.db)
+        s3.drop(testing.db)
+
+        # 't' is created, and 's3' won't be
+        # re-created since it's linked to 't'.
+        # 's1' and 's2' are, however.
+        metadata.create_all(testing.db)
+        assert self._has_sequence('s1')
+        assert self._has_sequence('s2')
+        assert not self._has_sequence('s3')
+
+        s2.drop(testing.db)
+        assert self._has_sequence('s1')
+        assert not self._has_sequence('s2')
+
+        metadata.drop_all(testing.db)
+        assert not self._has_sequence('s1')
+        assert not self._has_sequence('s2')
+
+
+class TableBoundSequenceTest(fixtures.TestBase):
+    __requires__ = ('sequences',)
+
+    @classmethod
+    def setup_class(cls):
+        global cartitems, sometable, metadata
+        metadata = MetaData(testing.db)
+        cartitems = Table("cartitems", metadata,
+            Column("cart_id", Integer, Sequence('cart_id_seq'), primary_key=True),
+            Column("description", String(40)),
+            Column("createdate", sa.DateTime())
+        )
+        sometable = Table( 'Manager', metadata,
+               Column('obj_id', Integer, Sequence('obj_id_seq'), ),
+               Column('name', String(128)),
+               Column('id', Integer, Sequence('Manager_id_seq', optional=True),
+                      primary_key=True),
+           )
+
+        metadata.create_all()
+
+    @classmethod
+    def teardown_class(cls):
+        metadata.drop_all()
+
+    def test_insert_via_seq(self):
+        cartitems.insert().execute(description='hi')
+        cartitems.insert().execute(description='there')
+        r = cartitems.insert().execute(description='lala')
+
+        assert r.inserted_primary_key and r.inserted_primary_key[0] is not None
+        id_ = r.inserted_primary_key[0]
+
+        eq_(1,
+            sa.select([func.count(cartitems.c.cart_id)],
+                      sa.and_(cartitems.c.description == 'lala',
+                              cartitems.c.cart_id == id_)).scalar())
+
+        cartitems.select().execute().fetchall()
+
     def test_seq_nonpk(self):
         """test sequences fire off as defaults on non-pk columns"""
 
@@ -630,63 +929,242 @@ class SequenceTest(testing.TestBase, testing.AssertsCompiledSQL):
              (3, "name3", 3),
              (4, "name4", 4)])
 
-    @testing.requires.sequences
-    def test_sequence(self):
-        cartitems.insert().execute(description='hi')
-        cartitems.insert().execute(description='there')
-        r = cartitems.insert().execute(description='lala')
 
-        assert r.inserted_primary_key and r.inserted_primary_key[0] is not None
-        id_ = r.inserted_primary_key[0]
+class SpecialTypePKTest(fixtures.TestBase):
+    """test process_result_value in conjunction with primary key columns.
 
-        eq_(1,
-            sa.select([func.count(cartitems.c.cart_id)],
-                      sa.and_(cartitems.c.description == 'lala',
-                              cartitems.c.cart_id == id_)).scalar())
+    Also tests that "autoincrement" checks are against column.type._type_affinity,
+    rather than the class of "type" itself.
 
-        cartitems.select().execute().fetchall()
-
-    @testing.fails_on('maxdb', 'FIXME: unknown')
-    # maxdb db-api seems to double-execute NEXTVAL internally somewhere,
-    # throwing off the numbers for these tests...
-    @testing.requires.sequences
-    def test_implicit_sequence_exec(self):
-        s = Sequence("my_sequence", metadata=MetaData(testing.db))
-        s.create()
-        try:
-            x = s.execute()
-            eq_(x, 1)
-        finally:
-            s.drop()
-
-    @testing.fails_on('maxdb', 'FIXME: unknown')
-    @testing.requires.sequences
-    def teststandalone_explicit(self):
-        s = Sequence("my_sequence")
-        s.create(bind=testing.db)
-        try:
-            x = s.execute(testing.db)
-            eq_(x, 1)
-        finally:
-            s.drop(testing.db)
-
-    @testing.requires.sequences
-    def test_checkfirst(self):
-        s = Sequence("my_sequence")
-        s.create(testing.db, checkfirst=False)
-        s.create(testing.db, checkfirst=True)
-        s.drop(testing.db, checkfirst=False)
-        s.drop(testing.db, checkfirst=True)
-
-    @testing.fails_on('maxdb', 'FIXME: unknown')
-    @testing.requires.sequences
-    def teststandalone2(self):
-        x = cartitems.c.cart_id.default.execute()
-        self.assert_(1 <= x <= 4)
+    """
 
     @classmethod
-    @testing.requires.sequences
-    def teardown_class(cls):
-        metadata.drop_all()
+    def setup_class(cls):
+        class MyInteger(TypeDecorator):
+            impl = Integer
+            def process_bind_param(self, value, dialect):
+                if value is None:
+                    return None
+                return int(value[4:])
+
+            def process_result_value(self, value, dialect):
+                if value is None:
+                    return None
+                return "INT_%d" % value
+
+        cls.MyInteger = MyInteger
+
+    @testing.provide_metadata
+    def _run_test(self, *arg, **kw):
+        metadata = self.metadata
+        implicit_returning = kw.pop('implicit_returning', True)
+        kw['primary_key'] = True
+        if kw.get('autoincrement', True):
+            kw['test_needs_autoincrement'] = True
+        t = Table('x', metadata,
+            Column('y', self.MyInteger, *arg, **kw),
+            Column('data', Integer),
+            implicit_returning=implicit_returning
+        )
+
+        t.create()
+        r = t.insert().values(data=5).execute()
+
+        # we don't pre-fetch 'server_default'.
+        if 'server_default' in kw and (not testing.db.dialect.implicit_returning or not implicit_returning):
+            eq_(r.inserted_primary_key, [None])
+        else:
+            eq_(r.inserted_primary_key, ['INT_1'])
+        r.close()
+
+        eq_(
+            t.select().execute().first(),
+            ('INT_1', 5)
+        )
+
+    def test_plain(self):
+        # among other things, tests that autoincrement
+        # is enabled.
+        self._run_test()
+
+    def test_literal_default_label(self):
+        self._run_test(default=literal("INT_1", type_=self.MyInteger).label('foo'))
+
+    def test_literal_default_no_label(self):
+        self._run_test(default=literal("INT_1", type_=self.MyInteger))
+
+    def test_sequence(self):
+        self._run_test(Sequence('foo_seq'))
+
+    def test_server_default(self):
+        self._run_test(server_default='1',)
+
+    def test_server_default_no_autoincrement(self):
+        self._run_test(server_default='1', autoincrement=False)
+
+    def test_clause(self):
+        stmt = select([literal("INT_1", type_=self.MyInteger)]).as_scalar()
+        self._run_test(default=stmt)
+
+    @testing.requires.returning
+    def test_no_implicit_returning(self):
+        self._run_test(implicit_returning=False)
+
+    @testing.requires.returning
+    def test_server_default_no_implicit_returning(self):
+        self._run_test(server_default='1', autoincrement=False)
+
+class ServerDefaultsOnPKTest(fixtures.TestBase):
+    @testing.provide_metadata
+    def test_string_default_none_on_insert(self):
+        """Test that without implicit returning, we return None for 
+        a string server default.
+
+        That is, we don't want to attempt to pre-execute "server_default"
+        generically - the user should use a Python side-default for a case
+        like this.   Testing that all backends do the same thing here.
+
+        """
+
+        metadata = self.metadata
+        t = Table('x', metadata, 
+                Column('y', String(10), server_default='key_one', primary_key=True),
+                Column('data', String(10)),
+                implicit_returning=False
+                )
+        metadata.create_all()
+        r = t.insert().execute(data='data')
+        eq_(r.inserted_primary_key, [None])
+        eq_(
+            t.select().execute().fetchall(),
+            [('key_one', 'data')]
+        )
+
+    @testing.requires.returning
+    @testing.provide_metadata
+    def test_string_default_on_insert_with_returning(self):
+        """With implicit_returning, we get a string PK default back no problem."""
+
+        metadata = self.metadata
+        t = Table('x', metadata, 
+                Column('y', String(10), server_default='key_one', primary_key=True),
+                Column('data', String(10))
+                )
+        metadata.create_all()
+        r = t.insert().execute(data='data')
+        eq_(r.inserted_primary_key, ['key_one'])
+        eq_(
+            t.select().execute().fetchall(),
+            [('key_one', 'data')]
+        )
+
+    @testing.provide_metadata
+    def test_int_default_none_on_insert(self):
+        metadata = self.metadata
+        t = Table('x', metadata, 
+                Column('y', Integer, 
+                        server_default='5', primary_key=True),
+                Column('data', String(10)),
+                implicit_returning=False
+                )
+        assert t._autoincrement_column is None
+        metadata.create_all()
+        r = t.insert().execute(data='data')
+        eq_(r.inserted_primary_key, [None])
+        if testing.against('sqlite'):
+            eq_(
+                t.select().execute().fetchall(),
+                [(1, 'data')]
+            )
+        else:
+            eq_(
+                t.select().execute().fetchall(),
+                [(5, 'data')]
+            )
+    @testing.provide_metadata
+    def test_autoincrement_reflected_from_server_default(self):
+        metadata = self.metadata
+        t = Table('x', metadata, 
+                Column('y', Integer, 
+                        server_default='5', primary_key=True),
+                Column('data', String(10)),
+                implicit_returning=False
+                )
+        assert t._autoincrement_column is None
+        metadata.create_all()
+
+        m2 = MetaData(metadata.bind)
+        t2 = Table('x', m2, autoload=True, implicit_returning=False)
+        assert t2._autoincrement_column is None
+
+    @testing.provide_metadata
+    def test_int_default_none_on_insert_reflected(self):
+        metadata = self.metadata
+        t = Table('x', metadata, 
+                Column('y', Integer, 
+                        server_default='5', primary_key=True),
+                Column('data', String(10)),
+                implicit_returning=False
+                )
+        metadata.create_all()
+
+        m2 = MetaData(metadata.bind)
+        t2 = Table('x', m2, autoload=True, implicit_returning=False)
+
+        r = t2.insert().execute(data='data')
+        eq_(r.inserted_primary_key, [None])
+        if testing.against('sqlite'):
+            eq_(
+                t2.select().execute().fetchall(),
+                [(1, 'data')]
+            )
+        else:
+            eq_(
+                t2.select().execute().fetchall(),
+                [(5, 'data')]
+            )
+
+    @testing.requires.returning
+    @testing.provide_metadata
+    def test_int_default_on_insert_with_returning(self):
+        metadata = self.metadata
+        t = Table('x', metadata, 
+                Column('y', Integer, 
+                        server_default='5', primary_key=True),
+                Column('data', String(10))
+                )
+
+        metadata.create_all()
+        r = t.insert().execute(data='data')
+        eq_(r.inserted_primary_key, [5])
+        eq_(
+            t.select().execute().fetchall(),
+            [(5, 'data')]
+        )
+
+class UnicodeDefaultsTest(fixtures.TestBase):
+    def test_no_default(self):
+        c = Column(Unicode(32))
+
+    def test_unicode_default(self):
+        # Py3K
+        #default = 'foo'
+        # Py2K
+        default = u'foo'
+        # end Py2K
+        c = Column(Unicode(32), default=default)
 
 
+    def test_nonunicode_default(self):
+        # Py3K
+        #default = b'foo'
+        # Py2K
+        default = 'foo'
+        # end Py2K
+        assert_raises_message(
+            sa.exc.SAWarning,
+            "Unicode column received non-unicode default value.",
+            Column,
+            Unicode(32),
+            default=default
+        )
