@@ -64,16 +64,16 @@ class SelectableTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiled
 
         assert s1.corresponding_column(scalar_select) is s1.c.foo
         assert s2.corresponding_column(scalar_select) is s2.c.foo
-    
+
     def test_label_grouped_still_corresponds(self):
         label = select([table1.c.col1]).label('foo')
         label2 = label.self_group()
-        
+
         s1 = select([label])
         s2 = select([label2])
         assert s1.corresponding_column(label) is s1.c.foo
         assert s2.corresponding_column(label) is s2.c.foo
-        
+
     def test_direct_correspondence_on_labels(self):
         # this test depends on labels being part
         # of the proxy set to get the right result
@@ -127,6 +127,14 @@ class SelectableTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiled
         sel3 = visitors.ReplacingCloningVisitor().traverse(sel2)
         assert sel3.corresponding_column(col) is sel3.c.foo
 
+    def test_with_only_generative(self):
+        s1 = table1.select().as_scalar()
+        self.assert_compile(
+            s1.with_only_columns([s1]),
+            "SELECT (SELECT table1.col1, table1.col2, "
+            "table1.col3, table1.colx FROM table1) AS anon_1"
+        )
+
 
     def test_select_on_table(self):
         sel = select([table1, table2], use_labels=True)
@@ -161,7 +169,6 @@ class SelectableTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiled
         criterion = a.c.col1 == table2.c.col2
         self.assert_(criterion.compare(j.onclause))
 
-
     def test_union(self):
 
         # tests that we can correspond a column in a Select statement
@@ -376,6 +383,135 @@ class SelectableTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiled
             [s.c.col1, s.corresponding_column(c2)]
         )
 
+    def test_from_list_deferred_constructor(self):
+        c1 = Column('c1', Integer)
+        c2 = Column('c2', Integer)
+
+        s = select([c1])
+
+        t = Table('t', MetaData(), c1, c2)
+
+        eq_(c1._from_objects, [t])
+        eq_(c2._from_objects, [t])
+
+        self.assert_compile(select([c1]), 
+                    "SELECT t.c1 FROM t")
+        self.assert_compile(select([c2]), 
+                    "SELECT t.c2 FROM t")
+
+    def test_from_list_deferred_whereclause(self):
+        c1 = Column('c1', Integer)
+        c2 = Column('c2', Integer)
+
+        s = select([c1]).where(c1==5)
+
+        t = Table('t', MetaData(), c1, c2)
+
+        eq_(c1._from_objects, [t])
+        eq_(c2._from_objects, [t])
+
+        self.assert_compile(select([c1]), 
+                    "SELECT t.c1 FROM t")
+        self.assert_compile(select([c2]), 
+                    "SELECT t.c2 FROM t")
+
+    def test_from_list_deferred_fromlist(self):
+        m = MetaData()
+        t1 = Table('t1', m, Column('x', Integer))
+
+        c1 = Column('c1', Integer)
+        s = select([c1]).where(c1==5).select_from(t1)
+
+        t2 = Table('t2', MetaData(), c1)
+
+        eq_(c1._from_objects, [t2])
+
+        self.assert_compile(select([c1]), 
+                    "SELECT t2.c1 FROM t2")
+
+    def test_from_list_deferred_cloning(self):
+        c1 = Column('c1', Integer)
+        c2 = Column('c2', Integer)
+
+        s = select([c1])
+        s2 = select([c2])
+        s3 = sql_util.ClauseAdapter(s).traverse(s2)
+
+        Table('t', MetaData(), c1, c2)
+
+        self.assert_compile(
+            s3,
+            "SELECT t.c2 FROM t"
+        )
+
+    def test_from_list_with_columns(self):
+        table1 = table('t1', column('a'))
+        table2 = table('t2', column('b'))
+        s1 = select([table1.c.a, table2.c.b])
+        self.assert_compile(s1, 
+                "SELECT t1.a, t2.b FROM t1, t2"
+            )
+        s2 = s1.with_only_columns([table2.c.b])
+        self.assert_compile(s2, 
+                "SELECT t2.b FROM t2"
+            )
+
+        s3 = sql_util.ClauseAdapter(table1).traverse(s1)
+        self.assert_compile(s3, 
+            "SELECT t1.a, t2.b FROM t1, t2"
+        )
+        s4 = s3.with_only_columns([table2.c.b])
+        self.assert_compile(s4, 
+            "SELECT t2.b FROM t2"
+        )
+
+    def test_from_list_warning_against_existing(self):
+        c1 = Column('c1', Integer)
+        s = select([c1])
+
+        # force a compile.
+        self.assert_compile(
+            s,
+            "SELECT c1"
+        )
+
+        Table('t', MetaData(), c1)
+
+        self.assert_compile(
+            s,
+            "SELECT t.c1 FROM t"
+        )
+
+    def test_from_list_recovers_after_warning(self):
+        c1 = Column('c1', Integer)
+        c2 = Column('c2', Integer)
+
+        s = select([c1])
+
+        # force a compile.
+        eq_(str(s), "SELECT c1")
+
+        @testing.emits_warning()
+        def go():
+            return Table('t', MetaData(), c1, c2)
+        t = go()
+
+        eq_(c1._from_objects, [t])
+        eq_(c2._from_objects, [t])
+
+        # 's' has been baked.  Can't afford
+        # not caching select._froms.
+        # hopefully the warning will clue the user
+        self.assert_compile(s, "SELECT t.c1 FROM t")
+        self.assert_compile(select([c1]), "SELECT t.c1 FROM t")
+        self.assert_compile(select([c2]), "SELECT t.c2 FROM t")
+
+    def test_label_gen_resets_on_table(self):
+        c1 = Column('c1', Integer)
+        eq_(c1._label, "c1")
+        Table('t1', MetaData(), c1)
+        eq_(c1._label, "t1_c1")
+
 class AnonLabelTest(fixtures.TestBase):
     """Test behaviors that we hope to change with [ticket:2168]."""
 
