@@ -130,6 +130,32 @@ class AdaptTest(fixtures.TestBase):
                         getattr(t2, k) == t1.__dict__[k] or \
                         t1.__dict__[k] is None
 
+    def test_python_type(self):
+        eq_(types.Integer().python_type, int)
+        eq_(types.Numeric().python_type, decimal.Decimal)
+        eq_(types.Numeric(asdecimal=False).python_type, float)
+        # Py3K
+        #eq_(types.LargeBinary().python_type, bytes)
+        # Py2K
+        eq_(types.LargeBinary().python_type, str)
+        # end Py2K
+        eq_(types.Float().python_type, float)
+        eq_(types.Interval().python_type, datetime.timedelta)
+        eq_(types.Date().python_type, datetime.date)
+        eq_(types.DateTime().python_type, datetime.datetime)
+        # Py3K
+        #eq_(types.String().python_type, unicode)
+        # Py2K
+        eq_(types.String().python_type, str)
+        # end Py2K
+        eq_(types.Unicode().python_type, unicode)
+        eq_(types.String(convert_unicode=True).python_type, unicode)
+
+        assert_raises(
+            NotImplementedError,
+            lambda: types.TypeEngine().python_type
+        )
+
     @testing.uses_deprecated()
     def test_repr(self):
         for typ in self._all_types():
@@ -173,6 +199,23 @@ class TypeAffinityTest(fixtures.TestBase):
         ]:
             eq_(t1._compare_type_affinity(t2), comp, "%s %s" % (t1, t2))
 
+    def test_decorator_doesnt_cache(self):
+        from sqlalchemy.dialects import postgresql
+
+        class MyType(TypeDecorator):
+            impl = CHAR
+
+            def load_dialect_impl(self, dialect):
+                if dialect.name == 'postgresql':
+                    return dialect.type_descriptor(postgresql.UUID())
+                else:
+                    return dialect.type_descriptor(CHAR(32))
+
+        t1 = MyType()
+        d = postgresql.dialect()
+        assert t1._type_affinity is String
+        assert t1.dialect_impl(d)._type_affinity is postgresql.UUID
+
 class PickleMetadataTest(fixtures.TestBase):
     def testmeta(self):
         for loads, dumps in picklers():
@@ -626,6 +669,7 @@ class UnicodeTest(fixtures.TestBase, AssertsExecutionResults):
                 ('mysql','oursql'),
                 ('mysql','zxjdbc'),
                 ('mysql','mysqlconnector'),
+                ('mysql','pymysql'),
                 ('sqlite','pysqlite'),
                 ('oracle','zxjdbc'),
                 ('oracle','cx_oracle'),
@@ -930,6 +974,20 @@ class EnumTest(fixtures.TestBase):
             {'id':4, 'someenum':'four'}
         )
 
+    def test_mock_engine_no_prob(self):
+        """ensure no 'checkfirst' queries are run when enums
+        are created with checkfirst=False"""
+
+        e = engines.mock_engine()
+        t = Table('t1', MetaData(),
+            Column('x', Enum("x", "y", name="pge"))
+        )
+        t.create(e, checkfirst=False)
+        # basically looking for the start of 
+        # the constraint, or the ENUM def itself,
+        # depending on backend.
+        assert "('x'," in e.print_sql()
+
 class BinaryTest(fixtures.TestBase, AssertsExecutionResults):
     __excluded_on__ = (
         ('mysql', '<', (4, 1, 1)),  # screwy varbinary types
@@ -1144,15 +1202,16 @@ class ExpressionTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiled
 
 
     @testing.fails_on('firebird', 'Data type unknown on the parameter')
+    @testing.fails_on('mssql', 'int is unsigned ?  not clear')
     def test_operator_adapt(self):
         """test type-based overloading of operators"""
 
         # test string concatenation
         expr = test_table.c.data + "somedata"
-        assert testing.db.execute(select([expr])).scalar() == "somedatasomedata"
+        eq_(testing.db.execute(select([expr])).scalar(), "somedatasomedata")
 
         expr = test_table.c.id + 15
-        assert testing.db.execute(select([expr])).scalar() == 16
+        eq_(testing.db.execute(select([expr])).scalar(), 16)
 
         # test custom operator conversion
         expr = test_table.c.avalue + 40
@@ -1160,17 +1219,17 @@ class ExpressionTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiled
 
         # value here is calculated as (250 - 40) / 10 = 21
         # because "40" is an integer, not an "avalue"
-        assert testing.db.execute(select([expr.label('foo')])).scalar() == 21
+        eq_(testing.db.execute(select([expr.label('foo')])).scalar(), 21)
 
         expr = test_table.c.avalue + literal(40, type_=MyCustomType)
 
         # + operator converted to -
         # value is calculated as: (250 - (40 * 10)) / 10 == -15
-        assert testing.db.execute(select([expr.label('foo')])).scalar() == -15
+        eq_(testing.db.execute(select([expr.label('foo')])).scalar(), -15)
 
         # this one relies upon anonymous labeling to assemble result
         # processing rules on the column.
-        assert testing.db.execute(select([expr])).scalar() == -15
+        eq_(testing.db.execute(select([expr])).scalar(), -15)
 
     def test_typedec_operator_adapt(self):
         expr = test_table.c.bvalue + "hi"
